package aggSim;

import java.util.ArrayList;
import java.util.Iterator;

import ronmodel.Mapping;
import ronmodel.ONNode;
import ronmodel.ObjectNet;

/**
 * @author Tony Modica
 * 
 */
public class NetMorphism extends ArrayList<Mapping> {

	/**
	 *
	 */
	private static final long serialVersionUID = -1640872183662355258L;

	public NetMorphism() {
	}

	public ONNode getSource(final int i) {
		return get(i).getSource();
	}

	public ONNode getTarget(final int i) {
		return get(i).getTarget();
	}

	public ObjectNet getDomain() {
		if (isEmpty()) {
			return null;
		}
		return getSource(0).getObjectNet();
	}

	public ObjectNet getCodomain() {
		if (isEmpty()) {
			return null;
		}
		return getTarget(0).getObjectNet();
	}

	public boolean containsSource(final ONNode node) {
		for (final Mapping mapping : this) {
			if (mapping.getSource() == node) {
				return true;
			}
		}
		return false;
	}

	public boolean containsTarget(final ONNode node) {
		for (final Mapping mapping : this) {
			if (mapping.getTarget() == node) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Basically, netMorphisms are like sets, but I did not find a set
	 * implementation for our needs. So I just adopted a set-like equality for
	 * Mappings with same source and target node.
	 */
	@Override
	public boolean equals(final Object o) {
		if (o == this) {
			return true;
		}
		if (!(o instanceof NetMorphism)) {
			return false;
		}
		final NetMorphism morphism = (NetMorphism) o;
		if (morphism.size() != size()) {
			return false;
		}

		final Iterator<Mapping> e = morphism.iterator();
		while (e.hasNext()) {
			if (!contains(e.next())) {
				return false;
			}
		}
		return true;
	}

	public boolean contains(final Mapping mapping) {
		for (final Mapping itMapping : this) {
			if (itMapping.getSource() == mapping.getSource()
					&& itMapping.getTarget() == mapping.getTarget()) {
				return true;
			}
		}
		return false;
	}

	public boolean contains(final NetMorphism morphism) {
		for (final Mapping mapping : morphism) {
			if (!contains(mapping)) {
				return false;
			}
		}
		return true;
	}

}
