package aggSim.analyzer.dialogs;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import ronmodel.ObjectNet;
import ronmodel.Rule;
import aggSim.analyzer.Conflict;

public class LocalPlaceConflictAnalysisDialog extends Dialog {

	private Shell dialogShell;
	private final HashMap<ObjectNet, ArrayList<Conflict>> net2transTransConflicts;
	private final HashMap<ObjectNet, HashMap<Rule, List<Conflict>>> net2ruleTransConflictsMap;
	private final HashMap<ObjectNet, HashMap<Rule, HashMap<Rule, List<Conflict>>>> net2ruleRuleConflictsMap;
	private final String hlPlaceName;

	public LocalPlaceConflictAnalysisDialog(
			final String hlPlaceName,
			final HashMap<ObjectNet, ArrayList<Conflict>> net2transTransConflicts,
			final HashMap<ObjectNet, HashMap<Rule, List<Conflict>>> net2ruleTransConflictsMap,
			final HashMap<ObjectNet, HashMap<Rule, HashMap<Rule, List<Conflict>>>> net2ruleRuleConflictsMap) {
		super(new Shell(Display.getDefault()), SWT.NULL);
		this.hlPlaceName = hlPlaceName;
		this.net2transTransConflicts = net2transTransConflicts;
		this.net2ruleTransConflictsMap = net2ruleTransConflictsMap;
		this.net2ruleRuleConflictsMap = net2ruleRuleConflictsMap;
		open();
	}

	public void open() {
		try {
			final Shell parent = getParent();
			dialogShell = new Shell(parent, SWT.DIALOG_TRIM
					| SWT.APPLICATION_MODAL);

			final RowLayout dialogShellLayout = new RowLayout(
					org.eclipse.swt.SWT.HORIZONTAL);
			dialogShellLayout.fill = true;
			dialogShell.setLayout(dialogShellLayout);

			for (final Entry<ObjectNet, ArrayList<Conflict>> entry : net2transTransConflicts
					.entrySet()) {
				{
					dialogShell.setText("Local place conflicts on '"
							+ hlPlaceName + "'");
					final ArrayList<Conflict> transTransConflicts = entry
							.getValue();
					final ObjectNet objectNet = entry.getKey();
					final HashMap<Rule, List<Conflict>> ruleTransConflictsMap = net2ruleTransConflictsMap
							.get(objectNet);
					final HashMap<Rule, HashMap<Rule, List<Conflict>>> ruleRuleConflictsMap = net2ruleRuleConflictsMap
							.get(objectNet);

					final Group netGroup = new Group(dialogShell, SWT.NONE);
					final RowLayout netGroupLayout = new RowLayout(
							org.eclipse.swt.SWT.VERTICAL);
					netGroupLayout.fill = true;
					netGroup.setLayout(netGroupLayout);
					netGroup.setText("Object net: " + objectNet.getName());
					{
						final Group compositeTransTransConflicts = new Group(
								netGroup, SWT.NONE);
						final RowLayout compositeTransTransConflictsLayout = new RowLayout(
								org.eclipse.swt.SWT.HORIZONTAL);
						compositeTransTransConflicts
								.setLayout(compositeTransTransConflictsLayout);
						{
							final Label labelTTC = new Label(
									compositeTransTransConflicts, SWT.NONE);
							labelTTC.setText("Transition/transition conflicts: "
									+ transTransConflicts.size());
						}
						{
							final Button buttonTTC = new Button(
									compositeTransTransConflicts, SWT.PUSH
											| SWT.CENTER);
							buttonTTC.setText("Show");
							buttonTTC.addMouseListener(new MouseAdapter() {
								@Override
								public void mouseUp(final MouseEvent evt) {
									final TransTransConflictDialog diag = new TransTransConflictDialog(
											objectNet, transTransConflicts);
									diag.setDoLayout(objectNet.eContainer() == null);
									diag.open();
								}
							});
							// disable Button if no conflicts have been found
							buttonTTC
									.setEnabled(!transTransConflicts.isEmpty());
						}
					}

					final Group compositeRuleTransConflicts = new Group(
							netGroup, SWT.NONE);
					final RowLayout compositeRuleTransConflictsLayout = new RowLayout(
							org.eclipse.swt.SWT.VERTICAL);
					compositeRuleTransConflicts
							.setLayout(compositeRuleTransConflictsLayout);
					compositeRuleTransConflicts
							.setText("Rule/Transitions combinations: "
									+ ruleTransConflictsMap.size());

					final Group compositeRuleRuleConflicts = new Group(
							netGroup, SWT.NONE);
					final RowLayout compositeRuleRuleConflictsLayout = new RowLayout(
							org.eclipse.swt.SWT.VERTICAL);
					compositeRuleRuleConflicts
							.setLayout(compositeRuleRuleConflictsLayout);
					int counter = 0;
					for (final Entry<Rule, HashMap<Rule, List<Conflict>>> entry2 : ruleRuleConflictsMap
							.entrySet()) {
						final HashMap<Rule, List<Conflict>> map = entry2
								.getValue();
						for (final List<Conflict> conflicts : map.values()) {
							counter += conflicts.size();
						}
					}
					compositeRuleRuleConflicts
							.setText("Rule/Rule combinations: " + counter);

					// generate rows for each rule conflicting with some
					// transition firing
					final RowLayout tempLayout = new RowLayout(
							org.eclipse.swt.SWT.HORIZONTAL);
					for (final Entry<Rule, List<Conflict>> entry2 : ruleTransConflictsMap
							.entrySet()) {
						final Rule rule1 = entry2.getKey();
						final List<Conflict> conflicts = entry2.getValue();
						final Composite tempComposite = new Composite(
								compositeRuleTransConflicts, SWT.NONE);
						tempComposite.setLayout(tempLayout);
						final Label tempLabel = new Label(tempComposite,
								SWT.NONE);
						tempLabel.setText("Rule " + rule1.getName()
								+ " vs. Transitions conflicts: "
								+ conflicts.size());
						final Button tempButton = new Button(tempComposite,
								SWT.PUSH | SWT.CENTER);
						tempButton.setText("Show");
						tempButton.addMouseListener(new MouseAdapter() {
							@Override
							public void mouseUp(final MouseEvent evt) {
								final RuleTransConflictDialog diag = new RuleTransConflictDialog(
										objectNet, rule1, conflicts);
								diag.setDoLayout(objectNet.eContainer() == null);
								diag.open();
							}
						});
						// disable Button if no conflicts have been found
						tempButton.setEnabled(!conflicts.isEmpty());
					}
					// generate rows for each rule conflicting with some
					// transition firing
					for (final Entry<Rule, HashMap<Rule, List<Conflict>>> entry2 : ruleRuleConflictsMap
							.entrySet()) {
						final Rule rule1 = entry2.getKey();
						final HashMap<Rule, List<Conflict>> innerMap = entry2
								.getValue();
						for (final Entry<Rule, List<Conflict>> entry3 : innerMap
								.entrySet()) {
							final Rule rule2 = entry3.getKey();
							final List<Conflict> conflicts = entry3.getValue();
							final Composite tempComposite = new Composite(
									compositeRuleRuleConflicts, SWT.NONE);
							tempComposite.setLayout(tempLayout);
							final Label tempLabel = new Label(tempComposite,
									SWT.NONE);
							tempLabel.setText("Rule " + rule1.getName()
									+ " vs. Rule " + rule2.getName()
									+ " conflicts: " + conflicts.size());
							final Button tempButton = new Button(tempComposite,
									SWT.PUSH | SWT.CENTER);
							tempButton.setText("Show");
							tempButton.addMouseListener(new MouseAdapter() {
								@Override
								public void mouseUp(final MouseEvent evt) {
									final RuleRuleConflictDialog diag = new RuleRuleConflictDialog(
											objectNet, rule1, rule2, conflicts);
									diag.setDoLayout(objectNet.eContainer() == null);
									diag.open();
								}
							});
							// disable Button if no conflicts have been found
							tempButton.setEnabled(!conflicts.isEmpty());
						}
					}
				}
			}
			dialogShell.layout();
			dialogShell.setLocation(getParent().toDisplay(100, 100));
			dialogShell.open();
			dialogShell.pack();
			final Display display = dialogShell.getDisplay();
			while (!dialogShell.isDisposed()) {
				if (!display.readAndDispatch()) {
					display.sleep();
				}
			}
		} catch (final Exception e) {
			e.printStackTrace();
		}
	}

}
