package muvitorkit.animation.demo;

import java.util.ArrayList;
import java.util.List;

import muvitorkit.animation.AnimatingCommand;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.ui.IWorkbenchPart;

import ronmodel.HLArc;
import ronmodel.ONArc;
import ronmodel.ObjectNet;

/**
 * @author Tony Modica
 * 
 */
public class AnimatingDemoAction1 extends SelectionAction {

	static public final String ID = "AnimatingDemoAction1";

	private EObject selectedModel;

	public AnimatingDemoAction1(final IWorkbenchPart part) {
		super(part);
		setDescription(ID);
		setText(ID);
		setId(ID);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 */
	@Override
	protected boolean calculateEnabled() {
		final List<?> selectedObjects = getSelectedObjects();
		if (selectedObjects.size() != 1) {
			return false;
		}
		final Object selectedObject = selectedObjects.get(0);
		if (!(selectedObject instanceof EditPart)) {
			return false;
		}
		final EditPart selectedEditPart = (EditPart) selectedObject;
		final Object selectedEditPartModel = selectedEditPart.getModel();
		if (!(selectedEditPartModel instanceof EObject)) {
			return false;
		}

		selectedModel = (EObject) selectedEditPartModel;
		return selectedModel.eContents().size() >= 2;
	}

	@Override
	public void run() {
		execute(new AnimatingDemoCommand1(selectedModel));
	}

	private static class AnimatingDemoCommand1 extends AnimatingCommand {

		public AnimatingDemoCommand1(final EObject model) {
			flushAfter = true;
			List<EObject> children = model.eContents();

			// angepasst an RON model: Arcs rausfiltern!
			children = new ArrayList<EObject>();
			for (final EObject elem : model.eContents()) {
				if (!(elem instanceof ONArc || elem instanceof HLArc)) {
					children.add(elem);
				}
			}

			final EObject movingChild = children.get(0);

			// initialize animated elements
			// initializeAnimatedElement(model, model.eContainer());
			// angepasst an RON model, da das RON das oberste angezeigte Modell
			// im Viewer ist
			/*
			 * angepasst an RON model, da das RON das oberste angezeigte Modell
			 * im Viewer ist. Es kann auch das RON ausgewhlt und somit die
			 * HL-Nodes animiert werden.
			 */
			final boolean modelIsObjectNet = model instanceof ObjectNet;
			if (modelIsObjectNet) {
				initializeAnimatedElement(model, model.eContainer()
						.eContainer());
			}

			initializeAnimatedElement(movingChild, model);
			// initial positions
			if (modelIsObjectNet) {
				specifyStep(model, model);
			}
			specifyStep(movingChild, movingChild);

			for (int actualStep = 1; actualStep < children.size(); actualStep++) {
				nextStep(2);
				specifyStep(movingChild, children.get(actualStep));
				if (modelIsObjectNet && actualStep == (children.size() - 1) / 2) {
					specifyStep(model, model, 4);
				}
			}
			if (modelIsObjectNet) {
				specifyStep(model, model);
			}

		}
	}

}
