package muvitorkit.dialogs;

import muvitorkit.ui.utils.SWTResourceManager;

import org.eclipse.draw2d.ConnectionLayer;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.Tool;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import roneditor.gef.actions.ron.RunAutoSimulationAction;
import roneditor.model.ONTransitionUtil;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;

/**
 * This code was edited or generated using CloudGarden's Jigloo
 * SWT/Swing GUI Builder, which is free for non-commercial
 * use. If Jigloo is being used commercially (ie, by a corporation,
 * company or business for any purpose whatever) then you
 * should purchase a license for each developer using Jigloo.
 * Please visit www.cloudgarden.com for details.
 * Use of Jigloo implies acceptance of these licensing terms.
 * A COMMERCIAL LICENSE HAS NOT BEEN PURCHASED FOR
 * THIS MACHINE, SO JIGLOO OR THIS CODE CANNOT BE USED
 * LEGALLY FOR ANY CORPORATE OR COMMERCIAL PURPOSE.
 */
/**
 * @author Tony Modica
 * 
 */
public abstract class AbstractFireDialog extends Dialog implements
		ISelectionChangedListener {

	private EditDomain editDomain;

	private ScrollingGraphicalViewer viewer;

	private final ObjectNet objectNet;

	private boolean canceled = false;
	private Button checkBoxRandom;

	private ONTransition transitionToFire;

	final private ONTransition randomActivatedTransition;

	// visual stuff
	private Shell dialogShell;
	private Composite compositeApplyLower;
	private Button buttonFireRandom;
	private Button buttonCancel;
	private Button buttonFire;
	private Composite compositeViewer;
	private Composite compositeNetUpper;

	public AbstractFireDialog(final ObjectNet objectNet) {
		super(new Shell(Display.getDefault()), SWT.NULL);
		this.objectNet = objectNet;
		randomActivatedTransition = ONTransitionUtil
				.getRandomActivated(objectNet.getNodes());
	}

	/**
	 * Builds the dialog by creating the widget hierarchy and calling
	 * {@link #init()}.
	 * 
	 * @see #init()
	 */
	public void open() {
		try {
			final Shell parent = getParent();
			dialogShell = new Shell(parent, SWT.DIALOG_TRIM
					| SWT.APPLICATION_MODAL | SWT.RESIZE);

			{
				// Register as a resource user - SWTResourceManager will
				// handle the obtaining and disposing of resources
				SWTResourceManager.registerResourceUser(dialogShell);
			}

			final GridLayout dialogShellLayout = new GridLayout();
			dialogShellLayout.makeColumnsEqualWidth = true;
			dialogShell.setLayout(dialogShellLayout);

			dialogShell.setText("Select transition to fire");
			// START >> compositeNetUpper
			final GridData compositeNetMiddleLData = new GridData();
			compositeNetMiddleLData.horizontalAlignment = GridData.CENTER;
			compositeNetMiddleLData.grabExcessHorizontalSpace = true;
			compositeNetUpper = new Composite(dialogShell, SWT.NONE);
			final GridLayout compositeNetMiddleLayout = new GridLayout();
			compositeNetMiddleLayout.numColumns = 2;
			compositeNetMiddleLayout.makeColumnsEqualWidth = true;
			compositeNetUpper.setLayout(compositeNetMiddleLayout);
			compositeNetUpper.setLayoutData(compositeNetMiddleLData);
			// END << compositeNetUpper
			// START >> compositeViewer
			final GridData compositeViewerLData = new GridData();
			compositeViewerLData.horizontalAlignment = GridData.FILL;
			compositeViewerLData.verticalAlignment = GridData.FILL;
			compositeViewerLData.grabExcessHorizontalSpace = true;
			compositeViewerLData.grabExcessVerticalSpace = true;
			compositeViewer = new Composite(dialogShell, SWT.NONE);
			final FillLayout compositeViewerLayout = new FillLayout(
					org.eclipse.swt.SWT.HORIZONTAL);
			compositeViewer.setLayout(compositeViewerLayout);
			compositeViewer.setLayoutData(compositeViewerLData);
			// END << compositeViewer
			// START >> compositeApplyLower
			compositeApplyLower = new Composite(dialogShell, SWT.NONE);
			final GridLayout compositeAppLayout = new GridLayout();
			compositeAppLayout.makeColumnsEqualWidth = false;
			compositeAppLayout.numColumns = 4;
			final GridData compositeApplyLowerLData = new GridData();
			compositeApplyLowerLData.horizontalAlignment = GridData.CENTER;
			compositeApplyLower.setLayoutData(compositeApplyLowerLData);
			compositeApplyLower.setLayout(compositeAppLayout);
			// START >> buttonFire
			buttonFire = new Button(compositeApplyLower, SWT.PUSH | SWT.CENTER);
			buttonFire.setText("Fire!");

			buttonFire.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseUp(final MouseEvent evt) {
					buttonFireMouseUp();
				}
			});
			// END << buttonFire
			{
				buttonFireRandom = new Button(compositeApplyLower, SWT.PUSH
						| SWT.CENTER);
				buttonFireRandom.setText("Fire randomly!");
				// disable if no random transition can be found
				if (randomActivatedTransition == null) {
					buttonFireRandom.setEnabled(false);
				} else {
					buttonFireRandom.addMouseListener(new MouseAdapter() {
						@Override
						public void mouseUp(final MouseEvent evt) {
							buttonFireRandomMouseUp(evt);
						}
					});
				}
			}
			{
				checkBoxRandom = new Button(compositeApplyLower, SWT.CHECK
						| SWT.LEFT);
				checkBoxRandom.setText("Fire randomly from now on");
				// this is only relevant for automatic simulations
				checkBoxRandom.setEnabled(RunAutoSimulationAction.isRunning());
				checkBoxRandom.addSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(final SelectionEvent evt) {
						checkBoxRandomWidgetSelected(evt);
					}
				});
			}
			// START >> buttonCancel
			buttonCancel = new Button(compositeApplyLower, SWT.PUSH
					| SWT.CENTER);
			final GridData buttonCancelLData = new GridData();
			buttonCancel.setLayoutData(buttonCancelLData);
			buttonCancel.setText("Cancel");
			buttonCancel.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseUp(final MouseEvent evt) {
					buttonCancelMouseUp();
				}
			});
			// END << buttonCancel
			// END << compositeApplyLower

			// set up the rest
			init();

			dialogShell.layout();
			dialogShell.pack();
			dialogShell.open();
			final Display display = dialogShell.getDisplay();
			while (!dialogShell.isDisposed()) {
				if (!display.readAndDispatch()) {
					display.sleep();
				}
			}
		} catch (final Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * 
	 */
	private void init() {
		createGraphicalViewer();
		viewer.setContents(objectNet);
		buttonFire.setEnabled(false);
	}

	/**
	 * Creates and configures a {@link ScrollingGraphicalViewer} on the
	 * sashForm.
	 * 
	 * @see #getEditDomain()
	 * @see #createEditPartFactory()
	 */
	private void createGraphicalViewer() {
		// set the new viewer on the viewer composite
		viewer = new ScrollingGraphicalViewer();
		viewer.createControl(compositeViewer);
		viewer.setEditDomain(getEditDomain());
		getEditDomain().addViewer(viewer);
		final ScalableFreeformRootEditPart rootEditPart = new ScalableFreeformRootEditPart();
		viewer.setRootEditPart(rootEditPart);
		viewer.setEditPartFactory(createEditPartFactory());
		// set antialias on connection layer, this may be turned off if
		// causing trouble
		((ConnectionLayer) rootEditPart
				.getLayer(LayerConstants.CONNECTION_LAYER))
				.setAntialias(SWT.ON);
		// some optional coloring
		viewer.getControl().setBackground(
				PlatformUI.getWorkbench().getDisplay()
						.getSystemColor(SWT.COLOR_LIST_BACKGROUND));
		viewer.getControl().setForeground(
				PlatformUI.getWorkbench().getDisplay()
						.getSystemColor(SWT.COLOR_LIST_FOREGROUND));
		viewer.addSelectionChangedListener(this);
	}

	/**
	 * @return An lazily created {@link EditPartFactory} for the
	 *         {@link GraphicalViewer} created in
	 *         {@link #createGraphicalViewer()}.
	 */
	abstract protected EditPartFactory createEditPartFactory();

	/**
	 * @return A lazily created {@link EditDomain} that works on this dialog's
	 *         command stack with a {@link Tool} for creating Mappings as
	 *         default tool.
	 * 
	 * @see #getCommandStack()
	 */
	private EditDomain getEditDomain() {
		if (null == editDomain) {
			editDomain = new EditDomain();
			editDomain.setDefaultTool(createTransitionSelectionTool());
			editDomain.loadDefaultTool();
		}
		return editDomain;
	}

	abstract protected Tool createTransitionSelectionTool();

	void buttonFireMouseUp() {
		closeDialog();
	}

	/**
	 * 
	 */
	void buttonCancelMouseUp() {
		RunAutoSimulationAction.setRunning(false);
		canceled = true;
		closeDialog();
	}

	/**
	 * 
	 */
	private void closeDialog() {
		dialogShell.close();
	}

	/**
	 * @return The transition that has been selected to be fired.
	 */
	public ONTransition getTransitionToFire() {
		return transitionToFire;
	}

	/**
	 * @return <code>true</code> if this dialog has been canceled.
	 */
	public boolean isCanceled() {
		return canceled;
	}

	@Override
	public void selectionChanged(final SelectionChangedEvent event) {
		final StructuredSelection sel = (StructuredSelection) event
				.getSelection();
		if (sel.size() == 1) {
			final Object model = ((EditPart) sel.getFirstElement()).getModel();
			if (model instanceof ONTransition
					&& isActivated((ONTransition) model)) {
				buttonFire.setEnabled(true);
				transitionToFire = (ONTransition) model;
			} else {
				buttonFire.setEnabled(false);
			}
		} else {
			buttonFire.setEnabled(false);
		}
	}

	/**
	 * This method decides if a transition is activated. Subclasses should
	 * implement this to adapt the dialog to respect specific properties like
	 * weight and capacity of post places.
	 * 
	 * @param transition
	 *            The transition to test for activation
	 * @return Whether the transition is activated or not
	 */
	abstract protected boolean isActivated(ONTransition transition);

	void buttonFireRandomMouseUp(final MouseEvent evt) {
		transitionToFire = randomActivatedTransition;
		canceled = false;
		closeDialog();
	}

	void checkBoxRandomWidgetSelected(final SelectionEvent evt) {
		RunAutoSimulationAction.setSemiAuto(!checkBoxRandom.getSelection());
	}
}
