package roneditor.gef.actions.ron;

import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbench;

import roneditor.RONTreeEditor;
import roneditor.gef.commands.ron.CreateHLPlaceCommand;
import roneditor.ui.wizards.CreateHLPlaceWizard;
import ronmodel.HLPlaceType;
import ronmodel.RON;
import ronmodel.RonmodelFactory;

public class CreateHLPlaceAction extends SelectionAction {

	static public final String ID = "Create HLPlace";
	private RON ron;

	public CreateHLPlaceAction(final RONTreeEditor part) {
		super(part);
		setId(CreateHLPlaceAction.ID);
		setText("Create HLPlace");
		setDescription("Create a place in the RON");
		setToolTipText("Create a place in the RON");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	@Override
	public void run() {
		// create dialog
		final Shell shell = getWorkbenchPart().getSite().getShell();
		final IWorkbench workbench = getWorkbenchPart().getSite()
				.getWorkbenchWindow().getWorkbench();

		final CreateHLPlaceWizard placeWizard = new CreateHLPlaceWizard();
		placeWizard.init(workbench, (IStructuredSelection) getSelection());
		final WizardDialog dialog = new WizardDialog(shell, placeWizard);
		dialog.create();
		dialog.open();

		if (dialog.getReturnCode() == Window.CANCEL) {
			return;
		}

		// fill command from editpart with dialog data
		final CreateHLPlaceCommand command = new CreateHLPlaceCommand(ron,
				RonmodelFactory.eINSTANCE.createHLPlace());
		command.setName(placeWizard.getPlaceName());
		if (placeWizard.isRule()) {
			command.setType(HLPlaceType.RULE_LITERAL);
		} else {
			command.setType(HLPlaceType.NET_LITERAL);
		}
		execute(command);
	}

	/*
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 */
	@Override
	protected boolean calculateEnabled() {
		final List<?> selectedObjects = getSelectedObjects();
		if (selectedObjects.size() != 1) {
			return false;
		}
		final Object selected = getSelectedObjects().get(0);
		if (selected instanceof EditPart) {
			final Object model = ((EditPart) selected).getModel();
			if (model instanceof RON) {
				ron = (RON) model;
				return true;
			}
		}
		return false;
	}
}
