package roneditor.gef.actions.ron;

import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbench;

import roneditor.RONTreeEditor;
import roneditor.gef.commands.ron.AddTokenCommand;
import roneditor.ui.wizards.CreateTokenWizard;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.ObjectNet;
import ronmodel.RonmodelFactory;
import ronmodel.Rule;
import ronmodel.Token;

/**
 * An action for the RON editor to create tokens (i.e. low-level nets or rules)
 * on places in a RON.
 */
public class CreateTokenAction extends SelectionAction {

	private HLPlace hlPlace;

	static public final String ID = "Create Token";

	public CreateTokenAction(final RONTreeEditor part) {
		super(part);
		setId(CreateTokenAction.ID);
		setText("Create Token");
		setDescription("Create token on the place");
		setToolTipText("Create token on the place");
	}

	/**
	 * The Action shows an {@link CreateTokenWizard} and asks for the name of
	 * the {@link Token}.
	 * 
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	@Override
	public void run() {
		final Shell shell = getWorkbenchPart().getSite().getShell();
		final IWorkbench workbench = getWorkbenchPart().getSite()
				.getWorkbenchWindow().getWorkbench();

		final CreateTokenWizard tokenWizard = new CreateTokenWizard();
		tokenWizard.init(workbench, (IStructuredSelection) getSelection());
		final WizardDialog dialog = new WizardDialog(shell, tokenWizard);
		dialog.create();
		dialog.open();
		if (dialog.getReturnCode() == Window.CANCEL) {
			return;
		}

		// create command with dialog data
		AddTokenCommand command;
		if (hlPlace.getType().equals(HLPlaceType.NET_LITERAL)) {
			final ObjectNet newToken = RonmodelFactory.eINSTANCE
					.createObjectNet();
			command = new AddTokenCommand(hlPlace, newToken);
		} else {
			final Rule newToken = RonmodelFactory.eINSTANCE.createRule();
			newToken.setLhs(RonmodelFactory.eINSTANCE.createLHS());
			newToken.setRhs(RonmodelFactory.eINSTANCE.createRHS());
			newToken.getNacs().add(RonmodelFactory.eINSTANCE.createNAC());
			command = new AddTokenCommand(hlPlace, newToken);
		}
		command.setName(tokenWizard.getTokenName());
		execute(command);
	}

	/*
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 */
	@Override
	protected boolean calculateEnabled() {
		final List<?> selectedObjects = getSelectedObjects();
		if (selectedObjects.size() != 1) {
			return false;
		}
		final Object selected = getSelectedObjects().get(0);
		if (selected instanceof EditPart) {
			final Object model = ((EditPart) selected).getModel();
			if (model instanceof HLPlace) {
				hlPlace = (HLPlace) model;
				return true;
			}
		}
		return false;
	}
}
