package roneditor.gef.actions.ron;

import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.ui.IWorkbenchPart;

import roneditor.gef.commands.ron.HLTransitionFireCommand;
import roneditor.gef.editparts.ron.HLTransitionEditPart;
import roneditor.gef.editparts.tree.RONTreeEditPart;
import ronmodel.HLTransition;
import ronmodel.HLTransitionKind;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;

/**
 * {@link SelectionAction} for choosing a {@link ONTransition} in an
 * {@link ObjectNet} that should be fired.
 */
public class FireDialogAction extends SelectionAction {

	public static final String ID = "roneditor.actions.FireAction";

	private static final String NAME = "Fire Dialog";

	public FireDialogAction(final IWorkbenchPart part) {
		super(part);
		setId(ID);
		setText(NAME);
	}

	@Override
	public void run() {
		final HLTransitionEditPart transitionEditpart = (HLTransitionEditPart) getSelectedObjects()
				.get(0);
		final HLTransition hltransition = transitionEditpart.getCastedModel();
		final HLTransitionFireCommand command = new HLTransitionFireCommand(
				hltransition, true);
		getCommandStack().execute(command);
	}

	/**
	 * The {@link SelectionAction} is enabled if the first selected object is a
	 * {@link RONTreeEditPart}.
	 */
	@Override
	protected boolean calculateEnabled() {
		if (getSelection() != null && !getSelection().isEmpty()) {
			if (getSelectedObjects().get(0) instanceof HLTransitionEditPart) {
				final HLTransitionEditPart transitionEditpart = (HLTransitionEditPart) getSelectedObjects()
						.get(0);
				final HLTransition t = transitionEditpart.getCastedModel();
				if (t.getKind().getValue() == HLTransitionKind.FIRE
						&& t.isActivated()) {
					return true;
				}
			}
		}
		return false;
	}
}
