package roneditor.gef.actions.ron;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.IWorkbenchPart;

import roneditor.gef.actions.RONEditorActionBarContributor;
import roneditor.gef.editparts.ron.HLTransitionEditPart;
import roneditor.model.HLTransitionUtil;
import ronmodel.HLTransition;

/**
 * Action for firing a RON's transition.
 */
public class FireHLTransitionAction extends SelectionAction {

	public static final String ID = RONEditorActionBarContributor.FIRE_TRANSITION_ACTION_ID;

	private HLTransitionEditPart transitionPart;

	public FireHLTransitionAction(final IWorkbenchPart part) {
		super(part);
		setId(ID);
		setText(RONEditorActionBarContributor.FIRE_TRANSITION_ACTION_LABEL);
		setDescription(RONEditorActionBarContributor.FIRE_TRANSITION_ACTION_DESC);
		setToolTipText(RONEditorActionBarContributor.FIRE_TRANSITION_ACTION_DESC);
	}

	/**
	 * Gets a Command and executes it.
	 * 
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	@Override
	public void run() {
		final HLTransition transition = transitionPart.getCastedModel();
		final Command command = HLTransitionUtil.createFireCommand(transition);
		if (command == null) {
			return;
		}
		// for using this action independently from an editor
		if (getWorkbenchPart() == null || getCommandStack() == null) {
			transitionPart.getRoot().getViewer().getEditDomain()
					.getCommandStack().execute(command);
		} else {
			execute(command);
		}

	}

	/**
	 * Check if one and only one {@link HLTransitionEditPart} is selected and
	 * verify it can fire.
	 * 
	 * @return true if we have selected a transition an if it can fire, false
	 *         otherwise.
	 * 
	 */
	@Override
	protected boolean calculateEnabled() {
		final StructuredSelection selection = (StructuredSelection) getSelection();

		if (selection == null || selection.size() != 1) {
			return false;
		}

		final Object selectionElement = selection.getFirstElement();
		if (!(selectionElement instanceof HLTransitionEditPart)) {
			return false;
		}

		transitionPart = (HLTransitionEditPart) selectionElement;
		return transitionPart.getCastedModel().isActivated();
	}

	/**
	 * This setter allows universal usage of this action. Just call the
	 * constructor with <code>null</code> and set the transitionPart manually.
	 * 
	 * @param transitionPart
	 */
	public void setTransitionPart(final HLTransitionEditPart transitionPart) {
		this.transitionPart = transitionPart;
	}

}
