package roneditor.gef.actions.ron;

import java.util.ArrayList;
import java.util.List;

import muvitorkit.ui.utils.GenericUtils;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.EditorPartAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.internal.Workbench;
import org.eclipse.ui.plugin.AbstractUIPlugin;

import roneditor.RONTreeEditor;
import roneditor.model.HLTransitionUtil;
import ronmodel.HLNode;
import ronmodel.HLTransition;
import ronmodel.RON;

/**
 * 
 * @author Tony Modica
 */
@SuppressWarnings("restriction")
public class RunAutoSimulationAction extends EditorPartAction {

	private static final String DESC = "Run automatic simulation of the model";

	public static final String ID = "RunAutoSimulationAction";

	private static final String RUNNING_PROPERTY = "isRunningProperty";

	private static final String ICON_PATH = "$nl$/icons/";

	private static final String INITIAL_ICON = "org.eclipse.jdt.debug.ui.icons.full.etool16/run_exc.gif";

	private static final ImageDescriptor image;

	private static boolean isRunning;

	private static boolean isSemiAuto;

	private boolean deadlock;

	static {
		image = AbstractUIPlugin.imageDescriptorFromPlugin("RONEditor",
				ICON_PATH + INITIAL_ICON);
	}

	public RunAutoSimulationAction(final IEditorPart editor) {
		super(editor, AS_PUSH_BUTTON);
		setRunning(false);
		setSemiAuto(true);
		deadlock = false;
		setDescription(DESC);
		setToolTipText(DESC);
		setId(ID);
		setImageDescriptor(image);
	}

	@Override
	public void run() {
		if (!isRunning()) {
			// "Run" button has been pushed
			setRunning(true);
			firePropertyChange(RUNNING_PROPERTY, Boolean.FALSE, Boolean.TRUE);
			setChecked(true);
			final Display display = Workbench.getInstance().getDisplay();
			while (isRunning() && !deadlock) {
				// run simulation step
				if (runSimulationStep()) {
					// this does the trick that the workbench still
					// responds during animation!
					while (display.readAndDispatch()) {
						;
					}
				} else {
					// mark deadlock if not successful
					deadlock = true;
					setRunning(false);
				}
			}
			// not running any more
			firePropertyChange(RUNNING_PROPERTY, Boolean.TRUE, Boolean.FALSE);
			// reset possibly changed semi-auto behavior
			setSemiAuto(true);
			setChecked(false);
			if (deadlock) {
				final MessageDialog dialog = new MessageDialog(Display
						.getDefault().getActiveShell(), "Deadlock", null,
						"Simulation stopped due to a deadlock configuration.",
						MessageDialog.WARNING, new String[] { "OK" }, 0);
				dialog.open();
				deadlock = false;
			}
		}
	}

	@Override
	protected boolean calculateEnabled() {
		return !isRunning();
	}

	protected boolean runSimulationStep() {
		final HLTransition trans = getRandomActivatedTransition();
		if (trans != null) {
			final Command command = HLTransitionUtil.createFireCommand(trans);
			execute(command);
			return true;
		}
		return false;
	}

	/**
	 * @return
	 */
	private HLTransition getRandomActivatedTransition() {
		final RON ron = (RON) ((RONTreeEditor) getEditorPart())
				.getPrimaryModelRoot();
		final List<HLTransition> transitions = new ArrayList<HLTransition>();
		for (final HLNode node : ron.getNodes()) {
			if (node instanceof HLTransition) {
				final HLTransition trans = (HLTransition) node;
				if (trans.isActivated()) {
					transitions.add(trans);
				}
			}
		}
		return GenericUtils.getRandom(transitions);
	}

	/**
	 * @return the isRunning
	 */
	public static boolean isRunning() {
		return isRunning;
	}

	/**
	 * @param isRunning
	 *            the isRunning to set
	 */
	public static void setRunning(final boolean isRunning) {
		RunAutoSimulationAction.isRunning = isRunning;
	}

	/**
	 * @param isSemiAuto
	 *            the isSemiAuto to set
	 */
	public static void setSemiAuto(final boolean isSemiAuto) {
		RunAutoSimulationAction.isSemiAuto = isSemiAuto;
	}

	/**
	 * @return the isSemiAuto
	 */
	public static boolean isSemiAuto() {
		return isSemiAuto;
	}
}
