package roneditor.gef.actions.rule;

import java.util.Collection;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.ui.actions.WorkbenchPartAction;
import org.eclipse.ui.IWorkbenchPart;

import roneditor.gef.commands.rule.UpdateMappingCommand;
import roneditor.ui.rule.RulePage;
import ronmodel.Mapping;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ObjectNet;
import ronmodel.RonmodelFactory;

/**
 * This abstract {@link WorkbenchPartAction} handles copies from the
 * <code>source</code> to <code>target</code> {@link ObjectNet}.
 */
abstract public class ObjectnetCopyAction extends WorkbenchPartAction {

	protected ObjectNet source;

	protected ObjectNet target;

	final private RulePage rulePage;

	public ObjectnetCopyAction(final IWorkbenchPart part,
			final RulePage rulePage) {
		super(part);
		this.rulePage = rulePage;
	}

	@Override
	public void run() {
		final List<EObject> copyCollection = (List<EObject>) EcoreUtil
				.copyAll(source.eContents());
		final CompoundCommand command = new CompoundCommand();

		command.add(new CreateFromCollectionCommand(copyCollection, target));

		for (final Object sourceNodeObject : source.getNodes()) {
			final ONNode sourceNode = (ONNode) sourceNodeObject;
			final int sourcePosition = source.eContents().indexOf(sourceNode);

			final ONNode targetNode = (ONNode) copyCollection
					.get(sourcePosition);

			final Mapping mapping = RonmodelFactory.eINSTANCE.createMapping();
			final UpdateMappingCommand updateMappingCommand = new UpdateMappingCommand();
			updateMappingCommand.setSource(sourceNode);
			updateMappingCommand.setTarget(targetNode);
			updateMappingCommand.setCodomain(target);
			updateMappingCommand.setMapping(mapping);
			updateMappingCommand.update();
			command.add(updateMappingCommand);

		}
		execute(command);
	}

	@Override
	protected boolean calculateEnabled() {
		return source != null && target != null;
	}

	public RulePage getRulePage() {
		return rulePage;
	}

	/**
	 * {@link Command} to add {@link ONNode} and {@link ONArc} stored in a
	 * {@link Collection} to an {@link ObjectNet}.
	 */
	static protected class CreateFromCollectionCommand extends Command {
		private static final String LABEL = "CreateFromCollection";

		private final ObjectNet objectNet;

		private final Collection<EObject> collection;

		public CreateFromCollectionCommand(
				final Collection<EObject> collection, final ObjectNet objectnet) {
			super(LABEL);
			objectNet = objectnet;
			this.collection = collection;
		}

		@Override
		public void execute() {
			for (final Object object : collection) {
				if (object instanceof ONNode) {
					((ONNode) object).setObjectNet(objectNet);
				} else if (object instanceof ONArc) {
					((ONArc) object).setObjectNet(objectNet);
				}
			}
		}

		@Override
		public void undo() {
			for (final Object object : collection) {
				if (object instanceof ONNode) {
					((ONNode) object).setObjectNet(null);
				} else if (object instanceof ONArc) {
					((ONArc) object).setObjectNet(null);
				}
			}
		}
	}

}
