package roneditor.gef.commands.objectnet;

import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.commands.Command;

import ronmodel.HLPlace;
import ronmodel.Token;

public class ChangeTokenOrderCommand extends Command {

	private static final String LABEL = "Move Token '";

	private final Token token;

	private final Token after;

	private int newPosition;

	private int oldPosition;

	private final EList<Token> tokens;

	/**
	 * Changes the order of {@link Token} within an {@link HLPlace}.
	 * 
	 * @param hlPlace
	 *            the {@link HLPlace} where the {@link Token}s are in
	 * @param token
	 *            the child being moved
	 * @param after
	 *            null or the Token that should be after (or to the right of)
	 *            the child being moved
	 */
	public ChangeTokenOrderCommand(final HLPlace hlPlace, final Token token,
			final Token after) {
		this.token = token;
		this.after = after;
		tokens = hlPlace.getTokens();
	}

	/**
	 * Execute the {@link Command}, so change the order within the {@link EList}
	 * of the {@link Token}
	 */
	@Override
	public void execute() {
		newPosition = tokens.size();
		if (after != null) {
			newPosition = tokens.indexOf(after);
		}

		oldPosition = tokens.indexOf(token);

		// we shift the index when we move down (as the item above is beeing
		// removed)
		if (newPosition > oldPosition) {
			newPosition--;
		}

		tokens.move(newPosition, oldPosition);

		setLabel(LABEL + token.getName() + "': " + oldPosition + "->"
				+ newPosition);
	}

	@Override
	public void undo() {
		tokens.move(oldPosition, newPosition);
	}

	@Override
	public void redo() {
		tokens.move(newPosition, oldPosition);
	}

}
