package roneditor.gef.commands.objectnet;

import org.eclipse.gef.commands.Command;

import ronmodel.ONNode;
import ronmodel.ObjectNet;

/**
 * Command for creating a generic node
 * 
 */
public class CreateNodeCommand extends Command {

	private final static String LABEL = "Create ONNode ";

	private final ONNode onNode;

	private int x;

	private int y;

	private final ObjectNet objectNet;

	private String name = "";

	/**
	 * Constructor
	 * 
	 * @param objectNet
	 * @param node
	 */
	public CreateNodeCommand(final ONNode node, final ObjectNet objectNet) {
		super(LABEL);
		onNode = node;
		this.objectNet = objectNet;
	}

	/**
	 * Entry point for command execution.
	 */
	@Override
	public void execute() {
		onNode.setName(name);
		onNode.setX(x);
		onNode.setY(y);
		onNode.setObjectNet(objectNet);
	}

	/**
	 * Redo command execution.
	 */
	@Override
	public void redo() {
		onNode.setObjectNet(objectNet);
	}

	/**
	 * Undo command execution.
	 */
	@Override
	public void undo() {
		onNode.setObjectNet(null);
	}

	/**
	 * @param x
	 *            the x to set
	 */
	public void setLocation(final int x, final int y) {
		this.x = x;
		this.y = y;
	}

	/**
	 * @param name
	 *            the name to set
	 */
	public void setName(final String name) {
		this.name = name;
		setLabel(LABEL + "'" + name + "'");
	}

	/**
	 * @return the onNode
	 */
	public ONNode getONNode() {
		return onNode;
	}

}
