package roneditor.gef.commands.objectnet;

import org.eclipse.gef.commands.Command;

import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ONPlace;
import ronmodel.ONTransition;

/**
 * The <code>ReconnectCommand</code> reconnects the given {@link ONPlace} and
 * {@link ONTransition} or vice versa.
 */
public class ReconnectCommand extends Command {
	private static final String LABEL = "Reconnect '";

	private ONNode oldSource;

	private ONNode oldTarget;

	private final ONNode source;

	private final ONNode target;

	private final ONArc arc;

	/**
	 * Constructor.
	 * 
	 * @param link
	 * @param source
	 * @param target
	 */
	public ReconnectCommand(final ONArc link, final ONNode source,
			final ONNode target) {
		super(LABEL + source.getName() + "' -> '" + target.getName() + "'");
		arc = link;
		this.source = source;
		this.target = target;
	}

	/**
	 * Entry point for the command execution.
	 */
	@Override
	public void execute() {
		oldSource = arc.getSource();
		oldTarget = arc.getTarget();
		arc.setSource(source);
		arc.setTarget(target);
	}

	/**
	 * Redo command execution.
	 */
	@Override
	public void redo() {
		arc.setSource(source);
		arc.setTarget(target);
	}

	/**
	 * Undo command execution.
	 */
	@Override
	public void undo() {
		arc.setSource(oldSource);
		arc.setTarget(oldTarget);
	}
}
