package roneditor.gef.commands.ron;

import org.eclipse.gef.commands.Command;

import ronmodel.HLPlace;
import ronmodel.Token;

/**
 * Command for adding a Token to a HLPlace. This is undoable.
 * 
 * @author Tony Modica
 */
public class AddTokenCommand extends Command {

	private static final String LABEL = "Create Token ";

	final private HLPlace place;

	private final Token token;

	private String name;

	public AddTokenCommand(final HLPlace place, final Token token) {
		super(LABEL);
		this.place = place;
		this.token = token;
	}

	/**
	 * Executes the command by adding the Net to the HLPlace.
	 * 
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	@Override
	public void execute() {
		token.setName(name);
		token.setPlace(place);
	}

	/**
	 * Undoes the command by removing the Token from the HLPlace.
	 * 
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	@Override
	public void undo() {
		token.setPlace(null);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	@Override
	public void redo() {
		token.setPlace(place);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.Command#canExecute()
	 */
	@Override
	public boolean canExecute() {
		return name != null;
	}

	/**
	 * @param name
	 *            the name to set
	 */
	public void setName(final String name) {
		this.name = name;
		setLabel(LABEL + "'" + name + "'");
	}
}
