package roneditor.gef.commands.ron;

import org.eclipse.gef.commands.Command;

import ronmodel.HLArc;
import ronmodel.HLNode;
import ronmodel.HLPlace;
import ronmodel.HLTransition;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.RON;

/**
 * Command for connecting an {@link ONPlace} with an {@link ONTransition}.
 */
public class ConnectHLNodesCommand extends Command {
	private static final String LABEL = "Connect '";

	private final HLNode source;

	private HLNode target;

	private final HLArc arc;

	/**
	 * Constructor.
	 * 
	 * @param arc
	 * @param source
	 */
	public ConnectHLNodesCommand(final HLNode source, final HLArc arc) {
		super(LABEL);
		this.source = source;
		this.arc = arc;
	}

	public boolean isSourcePlace() {
		return source instanceof HLPlace;
	}

	/**
	 * True if the command can be executed, false otherwise.
	 */
	@Override
	public boolean canExecute() {
		if (source == null || target == null || arc == null || source == target) {
			return false;
		}
		return true;
	}

	/**
	 * Entry point for the command execution.
	 */
	@Override
	public void execute() {
		final RON objectnet = target.getRon();
		arc.setRon(objectnet);
		arc.setSource(source);
		arc.setTarget(target);
	}

	/**
	 * Undo command execution.
	 */
	@Override
	public void undo() {
		arc.setRon(null);
		arc.setSource(null);
		arc.setTarget(null);
	}

	/**
	 * Set the given {@link HLTransition} as target.
	 * 
	 * @param target
	 */
	public void setTarget(final HLNode target) {
		this.target = target;
		setLabel(LABEL + source.getName() + "' -> '" + target.getName() + "'");
	}

	/**
	 * @return the source
	 */
	public HLNode getSource() {
		return source;
	}
}
