/**
 *
 */
package roneditor.gef.commands.ron;

import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.gef.commands.Command;

import ronmodel.NAC;
import ronmodel.RonmodelFactory;
import ronmodel.Rule;

/**
 * Simple Command to add a {@link NAC} to an existing {@link Rule}.
 */
public class CreateNACCommand extends Command {
	private static final String LABEL = "Create NAC";

	private static final String NACPrefix = "nac ";

	private final Rule rule;

	private NAC nac;

	public CreateNACCommand(final Rule rule) {
		super(LABEL);
		this.rule = rule;
	}

	@Override
	public boolean canExecute() {
		return true;
	}

	@Override
	public void execute() {
		nac = RonmodelFactory.eINSTANCE.createNAC();
		// compute highest free number for (autogenerated) nac names
		int freeNac = 1;
		final List<NAC> nacs = rule.getNacs();
		for (final NAC currentNac : nacs) {
			final String name = currentNac.getName();
			if (name != null) {
				final Pattern p = Pattern.compile(NACPrefix + "(\\d*)");
				final Matcher m = p.matcher(name);
				if (m.matches()) {
					final int i = Integer.parseInt(m.group(1));
					if (freeNac <= i) {
						freeNac = i + 1;
					}
				}
			}
		}
		nac.setName(NACPrefix + freeNac);
		nac.setRule(rule);
	}

	@Override
	public void redo() {
		nac.setRule(rule);
	}

	@Override
	public void undo() {
		nac.setRule(null);
	}

	public NAC getNac() {
		return nac;
	}

}
