package roneditor.gef.commands.ron;

import org.eclipse.gef.commands.Command;

import ronmodel.HLArc;
import ronmodel.HLNode;
import ronmodel.RON;

/**
 * Command for deleting a {@link HLArc} between two {@link HLNode}s.
 */
public class DeleteHLArcCommand extends Command {

	private static final String LABEL = "Delete HLArc '";

	final private HLArc arc;

	final private HLNode source;

	final private HLNode target;

	final private RON network;

	public DeleteHLArcCommand(final HLArc arc) {
		super(LABEL + arc.getSource().getName() + "' -> '"
				+ arc.getTarget().getName() + "'");
		this.arc = arc;
		source = arc.getSource();
		target = arc.getTarget();
		network = arc.getRon();
	}

	@Override
	public boolean canExecute() {
		return arc != null && arc.getRon() != null && arc.getSource() != null
				&& arc.getTarget() != null;
	}

	/**
	 * Execute the command.
	 */
	@Override
	public void execute() {
		/*
		 * FIXED when this Command is used in a DeleteHLNodeCommand while
		 * deleting several HLNodes, this arc may have been already deleted. In
		 * this case this command should do nothing! (the compound command
		 * verifies execution *before* deleting any arc)
		 */
		if (!canExecute()) {
			return;
		}
		arc.setRon(null);
		arc.setSource(null);
		arc.setTarget(null);
	}

	@Override
	public void redo() {
		execute();
	}

	/**
	 * Undo command execution.
	 */
	@Override
	public void undo() {
		arc.setRon(network);
		arc.setSource(source);
		arc.setTarget(target);
	}
}
