/**
 *
 */
package roneditor.gef.commands.ron;

import muvitorkit.commands.SimpleDeleteEObjectCommand;

import org.eclipse.gef.commands.CompoundCommand;

import roneditor.gef.commands.objectnet.DeleteNodeCommand;
import ronmodel.Mapping;
import ronmodel.NAC;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.Rule;

/**
 * {@link CompoundCommand} to delete a {@link NAC} from a {@link Rule}.
 */
public class DeleteNACCommand extends CompoundCommand {

	public static final String LABEL = "Delete NAC ";

	/**
	 * The command will delete a {@link NAC} and all {@link ONNode} in it. As
	 * the {@link DeleteNodeCommand} also deletes all attached {@link ONArc} and
	 * {@link Mapping} the consistency on undo is ensured.
	 * 
	 * @param nac
	 *            The {@link NAC} to delete.
	 */
	public DeleteNACCommand(final NAC nac) {
		super(LABEL + nac.getName());
		for (final ONNode node : nac.getNodes()) {
			add(new DeleteNodeCommand(node));
		}
		add(new SimpleDeleteEObjectCommand(nac));
	}

}
