package roneditor.gef.commands.ron;

import org.eclipse.gef.commands.Command;

import ronmodel.HLPlace;
import ronmodel.Token;

/**
 * Command for removing a token from its parent place. This is undoable.
 */
public class DeleteTokenCommand extends Command {

	private static final String LABEL = "Delete Token ' ";

	final private Token token;

	final private HLPlace parent;

	final private int oldPosition;

	public DeleteTokenCommand(final Token token) {
		super(LABEL + token.getName() + "'");
		this.token = token;
		parent = token.getPlace();
		oldPosition = parent.getTokens().indexOf(token);
	}

	/**
	 * The child gets removed from its parent. The container is stored for
	 * undoing purposes.
	 * 
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	@Override
	public void execute() {
		token.setPlace(null);
	}

	/**
	 * Undoes the command by re-adding the previously removed object to its
	 * former container that {@link #execute()} has saved.
	 * 
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	@Override
	public void undo() {
		token.setPlace(parent);
		final int currentPosition = parent.getTokens().indexOf(token);
		parent.getTokens().move(oldPosition, currentPosition);
	}
}
