package roneditor.gef.commands.ron;

import org.eclipse.gef.commands.Command;

import ronmodel.HLArc;
import ronmodel.HLNode;
import ronmodel.HLPlace;
import ronmodel.HLTransition;

/**
 * The <code>ReconnectCommand</code> reconnects the given {@link HLPlace} and
 * {@link HLTransition} or vice versa.
 */
public class ReconnectHLNodeCommand extends Command {
	private static final String LABEL = "Reconnect '";

	private HLNode oldSource;

	private HLNode oldTarget;

	private final HLNode source;

	private final HLNode target;

	private final HLArc arc;

	/**
	 * Constructor.
	 * 
	 * @param target
	 * @param source
	 * @param link
	 */
	public ReconnectHLNodeCommand(final HLArc link, final HLNode source,
			final HLNode target) {
		super(LABEL + source.getName() + "' -> '" + target.getName() + "'");
		arc = link;
		this.source = source;
		this.target = target;
	}

	/**
	 * Entry point for the command execution.
	 */
	@Override
	public void execute() {
		oldSource = arc.getSource();
		oldTarget = arc.getTarget();
		arc.setSource(source);
		arc.setTarget(target);
	}

	/**
	 * Redo command execution.
	 */
	@Override
	public void redo() {
		arc.setSource(source);
		arc.setTarget(target);
	}

	/**
	 * Undo command execution.
	 */
	@Override
	public void undo() {
		arc.setSource(oldSource);
		arc.setTarget(oldTarget);
	}
}
