/**
 *
 */
package roneditor.gef.commands.rule;

import org.eclipse.gef.commands.Command;

import ronmodel.Mapping;
import ronmodel.ONNode;
import ronmodel.ObjectNet;

/**
 * {@link Command} to delete a {@link Mapping} between two {@link ONNode}. The
 * <code>DeleteMappingCommand</code> needs a {@link ONNode} as source and
 * target, and a {@link ObjectNet} to refer to as codomain. The {@link Command}
 * when executed will remove the {@link Mapping} accordingly.
 * 
 * This {@link Command} should not be used, use {@link UpdateMappingCommand}
 * instead.
 */
public class DeleteMappingCommand extends Command {

	private static final String LABEL = "Delete Mapping '";

	private final Mapping mapping;

	private ObjectNet codomain;

	private ONNode source;

	private ONNode target;

	public DeleteMappingCommand(final Mapping mapping) {
		super(LABEL + mapping.getSource().getName() + "' -> '"
				+ mapping.getTarget().getName() + "'");
		this.mapping = mapping;
	}

	@Override
	public boolean canExecute() {
		return mapping != null && mapping.getCodomain() != null
				&& mapping.getSource() != null && mapping.getTarget() != null;
	}

	@Override
	public void execute() {
		/*
		 * FIX when this Command is used in a DeleteNodeCommand while deleting
		 * several ONNodes, this mapping may have been already deleted. In this
		 * case this command should do nothing! (the compound command verifies
		 * execution before deleting any mapping)
		 */
		if (!canExecute()) {
			return;
		}
		source = mapping.getSource();
		target = mapping.getTarget();
		codomain = mapping.getCodomain();

		mapping.setCodomain(null);
		mapping.setSource(null);
		mapping.setTarget(null);
	}

	@Override
	public void undo() {
		mapping.setSource(source);
		mapping.setTarget(target);
		mapping.setCodomain(codomain);
	}

	@Override
	public void redo() {
		execute();
	}
}
