package roneditor.gef.commands.rule;

import org.eclipse.gef.commands.CompoundCommand;

import ronmodel.Mapping;
import ronmodel.ONNode;
import ronmodel.ObjectNet;

/**
 * {@link CompoundCommand} to dynamically add, change and delete {@link Mapping}
 * s between two {@link ONNode}s. The <code>UpdateMappingCommand</code> needs a
 * {@link ONNode} as source and target, and a {{@link ObjectNet} to refer to as
 * codomain. It will ensure the injectivity of the {@link Mapping}s hence, it
 * will automatically remove any already existing {@link Mapping}s either from
 * the source or to the target {@link ONNode}.
 * 
 * @see DeleteMappingCommand
 * @see CreateMappingCommand
 */

public class UpdateMappingCommand extends CompoundCommand {

	private static final String LABEL = "Create Mapping ";

	private Mapping mapping;

	private ObjectNet codomain;

	private ONNode source;

	private ONNode target;

	public void update() {
		getCommands().clear();
		boolean createNewMapping = true;
		if (codomain == null || mapping == null) {
			return;
		}
		for (final Object mappingObject : codomain.getMappings()) {
			final Mapping map = (Mapping) mappingObject;
			if (map.getTarget() == target || map.getSource() == source) {
				// if source AND target are already set, only delete the
				// connection!
				createNewMapping = !(map.getSource() == source && map
						.getTarget() == target) && createNewMapping;
				final DeleteMappingCommand deleteMappingCommand = new DeleteMappingCommand(
						map);
				add(deleteMappingCommand);
				setLabel("Delete Mapping " + map.getSource().getName() + "->"
						+ map.getTarget().getName());
			}
		}
		if (createNewMapping) {
			final CreateMappingCommand createMappingCommand = new CreateMappingCommand(
					mapping, codomain, source, target);
			add(createMappingCommand);
			setLabel(LABEL + source.getName() + "->" + target.getName());
		}
	}

	public void setMapping(final Mapping mapping) {
		this.mapping = mapping;
	}

	public void setCodomain(final ObjectNet codomain) {
		this.codomain = codomain;
	}

	public void setSource(final ONNode source) {
		this.source = source;
	}

	public void setTarget(final ONNode target) {
		this.target = target;
	}

}
