package roneditor.gef.editparts.objectnet;

import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.gef.editparts.objectnet.policies.ONPlaceEditPolicy;
import roneditor.gef.figures.objectnet.PlaceFigure;
import roneditor.model.properties.ONPlacePropertySource;
import ronmodel.ONPlace;
import ronmodel.RonmodelPackage;

/**
 * EditPart for a ONPlaces.
 */
public class ONPlaceEditPart extends ONNodeEditPart<ONPlace> {

	/**
	 * Constructor
	 * 
	 * @param place
	 */
	public ONPlaceEditPart(final ONPlace place) {
		super(place);
	}

	/**
	 * Update the figure and refresh the display.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	@Override
	protected void refreshVisuals() {
		((PlaceFigure) getNodeFigure()).setTokens(getCastedModel().getTokens());
		super.refreshVisuals();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * roneditor.gef.editparts.objectnet.ONNodeEditPart#notifyChanged(org.eclipse
	 * .emf.common.notify.Notification)
	 */
	@Override
	public void notifyChanged(final Notification notification) {
		super.notifyChanged(notification);
		final int featureId = notification.getFeatureID(RonmodelPackage.class);
		switch (featureId) {
		case RonmodelPackage.ON_PLACE__CAPACITY:
		case RonmodelPackage.ON_PLACE__TOKENS:
			refreshVisuals();
			updateTransitionFiringStatus();
		default:
			break;
		}
	}

	/**
	 * Call refreshVisuals() on all connected ONTransitionEditParts to update
	 * their firing status. Also reset the ObjectNet as token on its hosting
	 * HLPlace to notify the adjacent HLTransitions. This covers only the cases
	 * that capacity or tokens are changed!
	 */
	@SuppressWarnings("unchecked")
	public void updateTransitionFiringStatus() {
		for (final ConnectionEditPart outgoing : (List<ConnectionEditPart>) getSourceConnections()) {
			((ONTransitionEditPart) outgoing.getTarget()).updateFireEnabled();
		}
		for (final ConnectionEditPart incoming : (List<ConnectionEditPart>) getTargetConnections()) {
			((ONTransitionEditPart) incoming.getSource()).updateFireEnabled();
		}
	}

	/**
	 * Create the PlaceFigure as figure for the EditPart
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	@Override
	protected IFigure createFigure() {
		return new PlaceFigure();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getContentPane()
	 */
	@Override
	public IFigure getContentPane() {
		return ((PlaceFigure) getFigure()).getContentPane();
	}

	/**
	 * Install the {@link ONPlace} specific edit policies (currently only the
	 * {@link ONPlaceEditPolicy}).
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	@Override
	protected void createEditPolicies() {
		super.createEditPolicies();

		// install the edit policy to handle connection creation
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE,
				new ONPlaceEditPolicy());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * muvitorkit.gef.editparts.AdapterGraphicalEditPart#createPropertySource()
	 */
	@Override
	protected IPropertySource createPropertySource() {
		return new ONPlacePropertySource(getCastedModel());
	}

}
