package roneditor.gef.editparts.objectnet.policies;

import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;

import roneditor.gef.commands.objectnet.ConnectONNodesCommand;
import roneditor.gef.commands.objectnet.ReconnectCommand;
import roneditor.gef.editparts.objectnet.ONTransitionEditPart;
import ronmodel.ONArc;
import ronmodel.ONPlace;
import ronmodel.ONTransition;

/**
 * {@link EditPolicy} for an {@link ONTransitionEditPart}.
 */
public class ONTransitionEditPolicy extends ONNodeEditPolicy {

	public ONTransitionEditPolicy() {
		super();
	}

	/**
	 * When ever a connection is completed, we update the provided
	 * {@link CreateConnectionRequest} with the target, hence the
	 * {@link ONTransition} of the {@link ONTransitionEditPart}.
	 * 
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCompleteCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	@Override
	protected Command getConnectionCompleteCommand(
			final CreateConnectionRequest request) {
		final Command command = request.getStartCommand();
		if (command instanceof ConnectONNodesCommand) {
			final ConnectONNodesCommand connectONNodesCommand = (ConnectONNodesCommand) command;
			if (connectONNodesCommand.isSourcePlace()
					&& !isConnectedByArc(connectONNodesCommand.getSource())) {
				connectONNodesCommand.setTarget(getCastedModel());
				return command;
			}
		}
		return super.getConnectionCompleteCommand(request);
	}

	@Override
	public ONTransition getCastedModel() {
		return (ONTransition) super.getCastedModel();
	}

	/**
	 * When ever a connection is created, we update the provided
	 * {@link CreateConnectionRequest} with the source, hence the
	 * {@link ONTransition} of the {@link ONTransitionEditPart}.
	 * 
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	@Override
	protected Command getConnectionCreateCommand(
			final CreateConnectionRequest request) {
		final Object requestingObject = request.getNewObject();
		if (requestingObject instanceof ONArc) {
			final ConnectONNodesCommand command = new ConnectONNodesCommand(
					getCastedModel(), (ONArc) requestingObject);
			request.setStartCommand(command);
			return command;
		}
		return super.getConnectionCreateCommand(request);
	}

	/**
	 * When ever a connection is changed, we update the provided
	 * {@link ReconnectRequest} with the target, hence the {@link ONTransition}
	 * of the {@link ONTransitionEditPart}.
	 * 
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	@Override
	protected Command getReconnectTargetCommand(final ReconnectRequest request) {
		final ONArc link = (ONArc) request.getConnectionEditPart().getModel();
		if (link.getSource() instanceof ONPlace
				&& link.getTarget() != getCastedModel()) {
			final ReconnectCommand command = new ReconnectCommand(link,
					link.getSource(), getCastedModel());
			return command;
		}
		return null;
	}

	/**
	 * When ever a connection is changed, we update the provided
	 * {@link ReconnectRequest} with the source, hence the {@link ONTransition}
	 * of the {@link ONTransitionEditPart}.
	 * 
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	@Override
	protected Command getReconnectSourceCommand(final ReconnectRequest request) {
		final ONArc link = (ONArc) request.getConnectionEditPart().getModel();
		if (link.getTarget() instanceof ONPlace
				&& link.getSource() != getCastedModel()) {
			final ReconnectCommand command = new ReconnectCommand(link,
					getCastedModel(), link.getTarget());
			return command;
		}
		return null;
	}
}
