package roneditor.gef.editparts.ron;

import muvitorkit.gef.editparts.AdapterConnectionEditPart;

import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.gef.editparts.ron.policies.HLArcEditPolicy;
import roneditor.gef.editpolicies.ArcEndpointEditPolicy;
import roneditor.gef.figures.ron.HLArcFigure;
import roneditor.model.properties.HLArcPropertySource;
import ronmodel.HLArc;
import ronmodel.RonmodelPackage;

/**
 * {@link EditPart} for a {@link HLArc}.
 */
public class HLArcEditPart extends AdapterConnectionEditPart<HLArc> {

	/**
	 * Constructor
	 * 
	 * @param link
	 */
	public HLArcEditPart(final HLArc link) {
		super(link);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	@Override
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.CONNECTION_ENDPOINTS_ROLE,
				new ArcEndpointEditPolicy());
		// Not used because we use automatic routing of connections
		// installEditPolicy(EditPolicy.CONNECTION_BENDPOINTS_ROLE, new
		// ArcSelectionHandlesEditPolicy());
		installEditPolicy(EditPolicy.CONNECTION_ROLE, new HLArcEditPolicy());

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractConnectionEditPart#createFigure()
	 */
	@Override
	protected IFigure createFigure() {
		return new HLArcFigure(getCastedModel().getName());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	@Override
	protected void refreshVisuals() {
		final Connection c = getConnectionFigure();
		((HLArcFigure) c).setName(getCastedModel().getName());
		super.refreshVisuals();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * muvitorkit.gef.editparts.AdapterConnectionEditPart#createPropertySource()
	 */
	@Override
	protected IPropertySource createPropertySource() {
		return new HLArcPropertySource(getCastedModel());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * muvitorkit.gef.editparts.AdapterConnectionEditPart#notifyChanged(org.
	 * eclipse.emf.common.notify.Notification)
	 */
	@Override
	public void notifyChanged(final Notification notification) {
		final int featureId = notification.getFeatureID(RonmodelPackage.class);
		switch (featureId) {
		case RonmodelPackage.HL_ARC__NAME:
			refreshVisuals();
			break;
		case RonmodelPackage.HL_ARC__SOURCE:
		case RonmodelPackage.HL_ARC__TARGET:
			refreshVisuals();
			break;
		}
	}
}
