package roneditor.gef.editparts.ron;

import muvitorkit.ui.utils.SWTResourceManager;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPolicy;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.RONIconUtil;
import roneditor.gef.actions.ron.FireHLTransitionAction;
import roneditor.gef.editparts.ron.policies.HLTransitionEditPolicy;
import roneditor.gef.figures.objectnet.TransitionFigure;
import roneditor.model.properties.HLTransitionPropertySource;
import ronmodel.HLTransition;
import ronmodel.HLTransitionKind;
import ronmodel.RonmodelPackage;

/**
 * EditPart for an HLTransition.
 */
public class HLTransitionEditPart extends HLNodeEditPart<HLTransition> {

	/**
	 * @param transition
	 */
	public HLTransitionEditPart(final HLTransition transition) {
		super(transition);
	}

	/**
	 * Overridden to fire transition.
	 */
	@Override
	protected void performOpen() {
		// FireHLTransitionAction is responsible for checking if the transition
		// is activated
		final FireHLTransitionAction fireAction = new FireHLTransitionAction(
				null);
		fireAction.setTransitionPart(this);
		fireAction.run();
	}

	/**
	 * Create the edit policies for the Transition
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	@Override
	protected void createEditPolicies() {
		super.createEditPolicies();

		// install the edit policy to handle connection creation
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE,
				new HLTransitionEditPolicy());
	}

	/**
	 * Update the color of the figure depending if the transition can fire. Then
	 * execute super method.
	 * 
	 * @see ron.HLNodeEditPart#refreshVisuals()
	 */
	@Override
	protected void refreshVisuals() {
		final TransitionFigure figure = (TransitionFigure) getFigure();
		if (!getCastedModel().isSetValid()
				|| !getCastedModel().isSetActivated()) {
			figure.setGlow(true);
		} else {
			if (!getCastedModel().isValid()) {
				figure.setColor(SWTResourceManager.getColor(250, 100, 100));
			} else if (!getCastedModel().isActivated()) {
				figure.setColor(SWTResourceManager.getColor(220, 220, 220));
			} else {
				switch (getCastedModel().getKind().getValue()) {
				case HLTransitionKind.APPLYRULE:
					figure.setColor(SWTResourceManager.getColor(65, 255, 82));
					break;
				case HLTransitionKind.FIRE:
					figure.setColor(SWTResourceManager.getColor(255, 226, 90));
					break;
				case HLTransitionKind.SPLIT:
					figure.setColor(SWTResourceManager.getColor(180, 220, 255));
					break;
				case HLTransitionKind.STANDARD:
					figure.setColor(SWTResourceManager.getColor(179, 126, 238));
					break;
				default:
				}
			}
			figure.setGlow(false);
		}
		super.refreshVisuals();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * muvitorkit.gef.editparts.AdapterGraphicalEditPart#notifyChanged(org.eclipse
	 * .emf.common.notify.Notification)
	 */
	@Override
	public void notifyChanged(final Notification notification) {
		final int featureId = notification.getFeatureID(RonmodelPackage.class);
		switch (featureId) {
		case RonmodelPackage.HL_NODE__OUTGOING:
			refreshSourceConnections();
			refreshVisuals();
			break;
		case RonmodelPackage.HL_NODE__INCOMING:
			refreshTargetConnections();
			refreshVisuals();
			break;
		case RonmodelPackage.HL_TRANSITION__VALID:
			refreshVisuals();
			break;
		case RonmodelPackage.HL_TRANSITION__ACTIVATED:
			refreshVisuals();
			break;
		default:
			refreshVisuals();
			break;
		}
		super.notifyChanged(notification);
	}

	/**
	 * Create the TransitionFigure as figure for the EditPart.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	@Override
	protected IFigure createFigure() {
		final TransitionFigure transitionFigure = new TransitionFigure();
		transitionFigure.setIcon(RONIconUtil.getIcon(RONIconUtil.HL_TRANSITION,
				getCastedModel().getKind().getValue(), 16));
		return transitionFigure;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * muvitorkit.gef.editparts.AdapterGraphicalEditPart#createPropertySource()
	 */
	@Override
	protected IPropertySource createPropertySource() {
		return new HLTransitionPropertySource(getCastedModel());
	}
}
