package roneditor.gef.editparts.ron;

import org.eclipse.core.runtime.Assert;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;

import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.HLTransition;
import ronmodel.RON;
import ronmodel.Token;

/**
 * Factory class for graphical edit parts. Here, the corresponding edit parts of
 * a network are created.
 */
public class RONEditPartFactory implements EditPartFactory {

	/**
	 * Create an instance of the requested EditPart in the given context.
	 * Register the eAdapters of the Model in the rulePage if applicable.
	 * 
	 * @param context
	 *            The context
	 * @param obj
	 *            The type from the model to create the EditPart for
	 * 
	 * @see org.eclipse.gef.EditPartFactory#createEditPart(org.eclipse.gef.EditPart,
	 *      java.lang.Object)
	 */
	@Override
	public EditPart createEditPart(final EditPart context, final Object obj) {
		if (obj instanceof RON) {
			return new RONEditPart((RON) obj);
		} else if (obj instanceof HLPlace) {
			return new HLPlaceEditPart((HLPlace) obj);
		} else if (obj instanceof HLTransition) {
			return new HLTransitionEditPart((HLTransition) obj);
		} else if (obj instanceof HLArc) {
			return new HLArcEditPart((HLArc) obj);
		} else if (obj instanceof Token) {
			return new SimpleTokenEditPart((Token) obj);
		}
		Assert.isTrue(obj == null,
				"RONEditPartFactory could not create an EditPart for the model element "
						+ obj);
		return null;
	}
}
