package roneditor.gef.editparts.ron.policies;

import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;

import roneditor.gef.commands.ron.ConnectHLNodesCommand;
import roneditor.gef.commands.ron.ReconnectHLNodeCommand;
import roneditor.gef.editparts.objectnet.ONPlaceEditPart;
import roneditor.gef.editparts.ron.HLPlaceEditPart;
import roneditor.model.HLTransitionUtil;
import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.HLTransition;
import ronmodel.ONPlace;

/**
 * {@link EditPolicy} for an {@link ONPlaceEditPart}.
 */
public class HLPlaceEditPolicy extends HLNodeEditPolicy {

	@Override
	protected HLPlace getCastedModel() {
		return (HLPlace) super.getCastedModel();
	}

	/**
	 * When ever a connection is completed, we update the provided
	 * {@link CreateConnectionRequest} with the target, hence the
	 * {@link HLPlace} of the {@link HLPlaceEditPart}.
	 * 
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCompleteCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	@Override
	protected Command getConnectionCompleteCommand(
			final CreateConnectionRequest request) {
		final Command command = request.getStartCommand();
		if (command instanceof ConnectHLNodesCommand) {
			final ConnectHLNodesCommand connectHLNodesCommand = (ConnectHLNodesCommand) command;
			if (!connectHLNodesCommand.isSourcePlace()
					&& !isConnectedByArc(connectHLNodesCommand.getSource())) {
				connectHLNodesCommand.setTarget(getCastedModel());
				return connectHLNodesCommand;
			}
		}
		return super.getConnectionCompleteCommand(request);
	}

	/**
	 * When ever a connection is created, we update the provided
	 * {@link CreateConnectionRequest} with the source, hence the
	 * {@link ONPlace} of the {@link ONPlaceEditPart}.
	 * 
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	@Override
	protected Command getConnectionCreateCommand(
			final CreateConnectionRequest request) {
		final Object requestingObject = request.getNewObject();
		if (requestingObject instanceof HLArc) {
			final ConnectHLNodesCommand command = new ConnectHLNodesCommand(
					getCastedModel(), (HLArc) requestingObject);
			request.setStartCommand(command);
			return command;
		}
		return super.getConnectionCreateCommand(request);
	}

	/**
	 * When ever a connection is changed, we update the provided
	 * {@link ReconnectRequest} with the target, hence the {@link HLPlace} of
	 * the {@link HLPlaceEditPart}.
	 * 
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	@Override
	protected Command getReconnectTargetCommand(final ReconnectRequest request) {
		final HLArc link = (HLArc) request.getConnectionEditPart().getModel();
		if (link.getSource() instanceof HLTransition
				&& link.getTarget() != getCastedModel()) {
			final ReconnectHLNodeCommand command = new ReconnectHLNodeCommand(
					link, link.getSource(), getCastedModel());
			return command;
		}
		return null;
	}

	/**
	 * When ever a connection is changed, we update the provided
	 * {@link ReconnectRequest} with the source, hence the {@link HLPlace} of
	 * the {@link HLPlaceEditPart}.
	 * 
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	@Override
	protected Command getReconnectSourceCommand(final ReconnectRequest request) {
		final HLArc link = (HLArc) request.getConnectionEditPart().getModel();
		if (link.getTarget() instanceof HLTransition
				&& link.getSource() != getCastedModel()) {
			final HLTransition targetTransition = (HLTransition) link
					.getTarget();
			if (getCastedModel().getType() == HLPlaceType.RULE_LITERAL
					&& !HLTransitionUtil
							.getIncomingRulePlaces(targetTransition).isEmpty()) {
				return null;
			}
			final ReconnectHLNodeCommand command = new ReconnectHLNodeCommand(
					link, getCastedModel(), targetTransition);
			return command;
		}
		return null;
	}
}
