package roneditor.gef.editparts.tree;

import java.util.List;

import muvitorkit.gef.directedit.IDirectEditPart;
import muvitorkit.gef.editparts.AdapterTreeEditPart;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPolicy;
import org.eclipse.jface.viewers.ICellEditorValidator;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.RONIconUtil;
import roneditor.gef.editparts.ron.policies.HLNodeComponentEditPolicy;
import roneditor.gef.editpolicies.HLPlaceTreeContainerEditPolicy;
import roneditor.model.properties.HLPlacePropertySource;
import ronmodel.HLPlace;
import ronmodel.RonmodelPackage;
import ronmodel.Token;

/**
 * TreeEditPart for HLPlaces.
 * 
 * @author Tony Modica
 */
public class HLPlaceTreeEditPart extends AdapterTreeEditPart<HLPlace> implements
		IDirectEditPart {

	/**
	 * @param model
	 */
	public HLPlaceTreeEditPart(final HLPlace model) {
		super(model);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractTreeEditPart#createEditPolicies()
	 */
	@Override
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.COMPONENT_ROLE,
				new HLNodeComponentEditPolicy());
		installEditPolicy(EditPolicy.TREE_CONTAINER_ROLE,
				new HLPlaceTreeContainerEditPolicy());
	}

	/**
	 * Returns a List of all Tokens on the HLPlace which is the model of this
	 * EditPart.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	@Override
	protected List<Token> getModelChildren() {
		return getCastedModel().getTokens();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractTreeEditPart#getImage()
	 */
	@Override
	protected Image getImage() {
		return RONIconUtil.getIcon(RonmodelPackage.HL_PLACE, getCastedModel()
				.getType().getValue(), 16);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see muvitorkit.gef.directedit.IDirectEditPart#getDirectEditFeatureID()
	 */
	@Override
	public int getDirectEditFeatureID() {
		return RonmodelPackage.HL_PLACE__NAME;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see muvitorkit.gef.directedit.IDirectEditPart#getDirectEditValidator()
	 */
	@Override
	public ICellEditorValidator getDirectEditValidator() {
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see muvitorkit.gef.editparts.AdapterTreeEditPart#createPropertySource()
	 */
	@Override
	protected IPropertySource createPropertySource() {
		return new HLPlacePropertySource(getCastedModel());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * muvitorkit.gef.editparts.AdapterTreeEditPart#notifyChanged(org.eclipse
	 * .emf.common.notify.Notification)
	 */
	@Override
	public void notifyChanged(final Notification notification) {
		final int featureId = notification.getFeatureID(RonmodelPackage.class);
		switch (featureId) {
		case RonmodelPackage.HL_PLACE__TOKENS:
			refreshChildren();
			break;
		default:
			break;
		}
	}
}
