package roneditor.gef.editparts.tree;

import muvitorkit.gef.directedit.IDirectEditPart;
import muvitorkit.gef.editparts.AdapterTreeEditPart;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPolicy;
import org.eclipse.jface.viewers.ICellEditorValidator;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.RONIconUtil;
import roneditor.gef.editparts.ron.policies.TokenComponentEditPolicy;
import roneditor.model.properties.ObjectNetPropertySource;
import ronmodel.ObjectNet;
import ronmodel.RonmodelPackage;

/**
 * TreeEditPart for ObjectNets.
 * 
 * @author Tony Modica
 */
public class ObjectNetTreeEditPart extends AdapterTreeEditPart<ObjectNet>
		implements IDirectEditPart {

	/**
	 * @param model
	 */
	public ObjectNetTreeEditPart(final ObjectNet model) {
		super(model);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractTreeEditPart#createEditPolicies()
	 */
	@Override
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.COMPONENT_ROLE,
				new TokenComponentEditPolicy());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractTreeEditPart#getImage()
	 */
	@Override
	protected Image getImage() {
		return RONIconUtil.getIcon(RonmodelPackage.OBJECT_NET, 16);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see muvitorkit.gef.directedit.IDirectEditPart#getDirectEditFeatureID()
	 */
	@Override
	public int getDirectEditFeatureID() {
		return RonmodelPackage.OBJECT_NET__NAME;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see muvitorkit.gef.directedit.IDirectEditPart#getDirectEditValidator()
	 */
	@Override
	public ICellEditorValidator getDirectEditValidator() {
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see muvitorkit.gef.editparts.AdapterTreeEditPart#createPropertySource()
	 */
	@Override
	protected IPropertySource createPropertySource() {
		return new ObjectNetPropertySource(getCastedModel());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * muvitorkit.gef.editparts.AdapterTreeEditPart#notifyChanged(org.eclipse
	 * .emf.common.notify.Notification)
	 */
	@Override
	public void notifyChanged(final Notification notification) {
		final int featureId = notification.getFeatureID(RonmodelPackage.class);
		switch (featureId) {
		case RonmodelPackage.OBJECT_NET__NODES:
			refreshChildren();
			break;
		default:
			break;
		}
	}

}
