package roneditor.gef.editparts.tree;

import java.util.List;

import muvitorkit.gef.directedit.IDirectEditPart;
import muvitorkit.gef.editparts.AdapterTreeEditPart;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.jface.viewers.ICellEditorValidator;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.RONIconUtil;
import roneditor.model.properties.RONPropertySource;
import ronmodel.HLNode;
import ronmodel.RON;
import ronmodel.RonmodelPackage;

/**
 * TreeEditPart for the top-level container item in the tree editor. This
 * provides an visual item in the tree editor and allows the execution of
 * actions to create HLPlaces and HLTransitions via context menu on this item.
 * 
 * @author Tony Modica
 */
public class RONTreeEditPart extends AdapterTreeEditPart<RON> implements
		IDirectEditPart {

	/**
	 * @param model
	 */
	public RONTreeEditPart(final RON model) {
		super(model);
	}

	/**
	 * Returns a List of all HLNodes in the RON which is the model of this
	 * EditPart.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	@Override
	protected List<HLNode> getModelChildren() {
		return getCastedModel().getNodes();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractTreeEditPart#getImage()
	 */
	@Override
	protected Image getImage() {
		return RONIconUtil.getIcon(RonmodelPackage.RON, 16);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see muvitorkit.gef.editparts.AdapterTreeEditPart#createPropertySource()
	 */
	@Override
	protected IPropertySource createPropertySource() {
		return new RONPropertySource(getCastedModel());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * muvitorkit.gef.editparts.AdapterTreeEditPart#notifyChanged(org.eclipse
	 * .emf.common.notify.Notification)
	 */
	@Override
	public void notifyChanged(final Notification notification) {
		final int featureId = notification.getFeatureID(RonmodelPackage.class);
		switch (featureId) {
		case RonmodelPackage.RON__NODES:
			refreshChildren();
			break;
		default:
			break;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see muvitorkit.gef.directedit.IDirectEditPart#getDirectEditFeatureID()
	 */
	@Override
	public int getDirectEditFeatureID() {
		return RonmodelPackage.RON__NAME;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see muvitorkit.gef.directedit.IDirectEditPart#getDirectEditValidator()
	 */
	@Override
	public ICellEditorValidator getDirectEditValidator() {
		return null;
	}
}
