package roneditor.gef.editparts.tree;

import org.eclipse.core.runtime.Assert;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;

import ronmodel.HLPlace;
import ronmodel.HLTransition;
import ronmodel.ObjectNet;
import ronmodel.RON;
import ronmodel.Rule;

/**
 * Factory class for tree edit parts. Here, the corresponding edit parts of a
 * the model elements are created.
 * 
 * @author Tony Modica
 */
public class RONTreeEditPartFactory implements EditPartFactory {

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.gef.EditPartFactory#createEditPart(org.eclipse.gef.EditPart,
	 * java.lang.Object)
	 */
	@Override
	public EditPart createEditPart(final EditPart context, final Object obj) {
		if (obj instanceof RON) {
			return new RONTreeEditPart((RON) obj);
		} else if (obj instanceof HLPlace) {
			return new HLPlaceTreeEditPart((HLPlace) obj);
		} else if (obj instanceof HLTransition) {
			return new HLTransitionTreeEditPart((HLTransition) obj);
		} else if (obj instanceof ObjectNet) {
			return new ObjectNetTreeEditPart((ObjectNet) obj);
		} else if (obj instanceof Rule) {
			return new RuleTreeEditPart((Rule) obj);
		}

		Assert.isTrue(obj == null,
				"TreeEditPartFactory could not create an EditPart for the model element "
						+ obj);
		return null;
	}
}
