package roneditor.gef.editpolicies;

import java.util.ArrayList;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.LayoutEditPolicy;
import org.eclipse.gef.editpolicies.TreeContainerEditPolicy;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.eclipse.gef.requests.CreateRequest;

import roneditor.gef.commands.ron.MoveMultipleTokenCommand;
import roneditor.gef.editparts.tree.HLPlaceTreeEditPart;
import ronmodel.HLPlace;
import ronmodel.Token;

/**
 * {@link LayoutEditPolicy} for the {@link HLPlaceTreeEditPart}, used to support
 * drag and drop of {@link Token}.
 */
public class HLPlaceTreeContainerEditPolicy extends TreeContainerEditPolicy {

	/**
	 * We extract the models from the added editparts and return a
	 * {@link MoveMultipleTokenCommand} that handles the actual model change.
	 */
	@Override
	protected Command getAddCommand(final ChangeBoundsRequest request) {
		// Extract the Model Objects from the EditParts
		final ArrayList<Token> modelList = new ArrayList<Token>();
		for (final Object editPart : request.getEditParts()) {
			final Object model = ((EditPart) editPart).getModel();
			if (model instanceof Token) {
				modelList.add((Token) model);
			} else {
				return null;
			}
		}
		return new MoveMultipleTokenCommand(modelList, getCastedModel());
	}

	/**
	 * @return The model of the host {@link EditPart} casted as {@link HLPlace}.
	 */
	private HLPlace getCastedModel() {
		return (HLPlace) getHost().getModel();
	}

	@Override
	protected Command getMoveChildrenCommand(final ChangeBoundsRequest request) {
		return null;
	}

	@Override
	protected Command getCreateCommand(final CreateRequest request) {
		return null;
	}

}
