package roneditor.gef.figures.objectnet;

import muvitorkit.ui.utils.SWTResourceManager;

import org.eclipse.draw2d.Ellipse;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;

public class Orb extends Ellipse {

	/**
	 * Fill the shape with a Web2.0 like look, hence a radial gradient from
	 * foreground to background color, overlayed with an alpha-blended linear
	 * gradient circle. When we are in XOR mode (i.e. dragging the figure
	 * around), just display the original ellipse.
	 */
	@Override
	protected void fillShape(final Graphics graphics) {

		// when dragging, do a simple fill
		if (graphics.getXORMode()) {
			super.fillShape(graphics);
			return;
		}

		// enable anti-aliasing
		graphics.setAntialias(SWT.OFF);

		// draw circular gradient
		final Rectangle rect = getBounds();

		final int x = rect.x + rect.width / 2;
		final int y = rect.y + rect.width / 2;
		final float[] hsv = graphics.getBackgroundColor().getRGB().getHSB();

		final int radius = rect.width;
		for (int i = radius; i > 1; i--) {
			final Color color = SWTResourceManager.getColor(new RGB(hsv[0],
					hsv[1], hsv[2] - hsv[2] * i / (radius * 4.0f)));
			graphics.setForegroundColor(color);
			graphics.setBackgroundColor(color);
			graphics.fillOval(x - i / 2, y - i / 2, i, i);
		}

		// draw high-light as a white linear gradient from opaque to transparent
		final int radius2 = 2 * radius / 5;
		final int y2 = y - radius / 6;
		final int x2 = x;

		graphics.setForegroundColor(SWTResourceManager.getColor(255, 255, 255));
		graphics.setBackgroundColor(SWTResourceManager.getColor(255, 255, 255));
		final int counter = y2 + radius2;
		int alpha = 255;
		for (int count = y2 - radius2; count <= counter; count++) {
			alpha -= 255 / (radius2 * 2);
			final int radius2sqr = radius2 * radius2;
			final int offset1 = (int) (Math.sqrt(radius2sqr - (count - y2)
					* (count - y2)) + 0.5);
			final int offset2 = (int) (Math.sqrt(radius2sqr - (count - y2 + 1)
					* (count - y2 + 1)) + 0.5);
			graphics.setAlpha(alpha);
			graphics.fillPolygon(new int[] { x2 - offset1, count, x2 + offset1,
					count, x2 + offset2, count + 1, x2 - offset2, count + 1 });
		}
		graphics.setAlpha(255);
	}

	/**
	 * Enable anti-aliasing, ensure the foreground color is correct.
	 */
	@Override
	protected void outlineShape(final Graphics graphics) {
		graphics.setAntialias(SWT.ON);
		graphics.setForegroundColor(getForegroundColor());
		super.outlineShape(graphics);
	}
}
