package roneditor.gef.figures.objectnet;

import muvitorkit.gef.editparts.policies.IGhostFigureProvider;
import muvitorkit.ui.utils.SWTResourceManager;

import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Ellipse;
import org.eclipse.draw2d.FigureUtilities;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Color;

/**
 * Represents the graphical layout of a transition.
 */
public class PlaceFigure extends NodeFigure implements IGhostFigureProvider {
	protected final Label tokenLabel;

	protected final Ellipse placeEllipse;

	protected int tokens;

	private int radius = 30;

	private static final Color defaultColor = SWTResourceManager.getColor(224,
			229, 255);

	public PlaceFigure() {
		setOutline(false);
		setFill(false);
		tokens = 0;

		placeEllipse = new Orb();
		placeEllipse.setBackgroundColor(defaultColor);
		placeEllipse.setFill(true);
		add(placeEllipse);

		tokenLabel = new Label(" ");
		add(tokenLabel);

		incomingConnectionAnchor = new ChopboxAnchor(placeEllipse);
		outgoingConnectionAnchor = new ChopboxAnchor(placeEllipse);
	}

	@Override
	public void setGlow(final boolean glow) {
		if (glow) {
			placeEllipse.setForegroundColor(ColorConstants.red);
		} else {
			placeEllipse.setForegroundColor(ColorConstants.black);
		}
		super.setGlow(glow);
	}

	private Rectangle getEllipseBounds() {
		return new Rectangle(getDiameter() / 2 - radius / 2, 0, radius, radius);
	}

	private Rectangle getLabelBounds() {
		final Dimension d = nameLabel.getPreferredSize();
		return new Rectangle((getDiameter() - d.width) / 2, radius + 4, -1, -1);
	}

	private Rectangle getTokenLabelBounds() {
		final Dimension d = tokenLabel.getPreferredSize();
		return new Rectangle((getDiameter() - d.width) / 2,
				1 + (radius - d.height) / 2, -1, -1);
	}

	public void setTokens(final int tokens) {
		this.tokens = tokens;
		if (tokens > 0) {
			tokenLabel.setText(String.valueOf(tokens));
		} else {
			tokenLabel.setText("");
		}
		updateLayout();
	}

	// public void makeGhostShape() {
	// FigureUtilities.makeGhostShape(placeEllipse);
	// placeEllipse.setLineStyle(Graphics.LINE_DOT);
	// placeEllipse.setForegroundColor(ColorConstants.white);
	// setFill(false);
	// setLineWidth(0);
	// setOutline(false);
	// }

	private int getDiameter() {
		return Math.max(nameLabel.getPreferredSize().width, radius);
	}

	private void updateLayout() {
		getLayoutManager().setConstraint(nameLabel, getLabelBounds());
		getLayoutManager().setConstraint(placeEllipse, getEllipseBounds());
		getLayoutManager().setConstraint(tokenLabel, getTokenLabelBounds());
	}

	@Override
	public void validate() {
		updateLayout();
		super.validate();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.draw2d.IFigure#getMinimumSize(int, int)
	 */
	@Override
	public Dimension getMinimumSize(final int wHint, final int hHint) {
		return getPreferredSize();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.draw2d.IFigure#getPreferredSize(int, int)
	 */
	@Override
	public Dimension getPreferredSize(final int wHint, final int hHint) {
		final int diam = Math.max(nameLabel.getSize().width, 40);
		return new Dimension(diam, 50);
	}

	/**
	 * Sets the new size
	 * 
	 * @param width
	 *            the new width
	 * @param height
	 *            the new height
	 * 
	 */
	@Override
	public void setSize(final int width, final int height) {
		// calculate the radius
		radius = Math.round((float) Math.sqrt(width * width + height * height));
		updateLayout();
	}

	@Override
	public void setColor(final Color c) {
		placeEllipse.setBackgroundColor(c);
	}

	@Override
	public Color getColor() {
		return placeEllipse.getBackgroundColor();
	}

	public IFigure getContentPane() {
		return placeEllipse;
	}

	/**
	 * The {@link PlaceGhostFigure} is used when dragging the PlaceFigure.
	 */
	static private class PlaceGhostFigure extends PlaceFigure {

		PlaceGhostFigure(final PlaceFigure placeFigure) {
			FigureUtilities.makeGhostShape(placeEllipse);
			placeEllipse.setForegroundColor(ColorConstants.gray);
			tokens = placeFigure.tokens;
			if (tokens > 0) {
				tokenLabel.setText(String.valueOf(tokens));
			} else {
				tokenLabel.setText("");
			}
			tokenLabel.setForegroundColor(ColorConstants.gray);
			nameLabel.setText(placeFigure.nameLabel.getText());
			nameLabel.setForegroundColor(ColorConstants.gray);
		}
	}

	@Override
	public IFigure getGhostFigure() {
		return new PlaceGhostFigure(this);
	}
}