package roneditor.model;

import org.eclipse.core.runtime.Assert;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.requests.CreationFactory;

import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.HLTransition;
import ronmodel.Mapping;
import ronmodel.ONArc;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.RonmodelFactory;
import ronmodel.Token;

/**
 * This class implements the CreationFactory used by the CreationTool. It uses
 * the EMF-generated factories to create the model instances.
 * 
 * @author Tony Modica
 */
public class ModelCreationFactory implements CreationFactory {

	final private Class<?> targetClass;

	public ModelCreationFactory(final Class<?> targetClass) {
		this.targetClass = targetClass;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.requests.CreationFactory#getNewObject()
	 */
	@Override
	public Object getNewObject() {
		EObject result = null;
		final RonmodelFactory factory = RonmodelFactory.eINSTANCE;
		if (targetClass.equals(ONTransition.class)) {
			result = factory.createONTransition();
			((ONTransition) result).setColor(factory.createNodeColor());
		} else if (targetClass.equals(ONPlace.class)) {
			result = factory.createONPlace();
			((ONPlace) result).setColor(factory.createNodeColor());
		} else if (targetClass.equals(ONArc.class)) {
			result = factory.createONArc();
		} else if (targetClass.equals(Mapping.class)) {
			result = factory.createMapping();
		} else if (targetClass.equals(HLPlace.class)) {
			result = factory.createHLPlace();
		} else if (targetClass.equals(HLArc.class)) {
			result = factory.createHLArc();
		} else if (targetClass.equals(HLTransition.class)) {
			result = factory.createHLTransition();
		} else if (targetClass.equals(Token.class)) {
			/*
			 * we can safely return here because the HLPlaceFlowLayoutEditPolicy
			 * will take care about creating the appropriate model (objectnet or
			 * rule). Token is considered an abstract class!
			 */
			return null;
		}
		Assert.isNotNull(result,
				"ModelCreationFactory could not create a model!");
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.requests.CreationFactory#getObjectType()
	 */
	@Override
	public Object getObjectType() {
		return targetClass;
	}
}
