package roneditor.model.properties;

import java.util.ArrayList;

import muvitorkit.properties.AbstractPropertySource;
import muvitorkit.properties.LocationPropertySource;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import ronmodel.HLTransition;
import ronmodel.HLTransitionKind;

/**
 * Property source for a HLTransition.
 */
public class HLTransitionPropertySource extends
		AbstractPropertySource<HLTransition> {

	private static enum ID {
		NAME, KIND, VALID, ENABLED, LOCATION
	}

	public HLTransitionPropertySource(final HLTransition hlTransition) {
		super(hlTransition);
	}

	@Override
	protected IPropertyDescriptor[] createPropertyDescriptors() {
		final ArrayList<IPropertyDescriptor> descriptorList = new ArrayList<IPropertyDescriptor>();
		descriptorList.add(new TextPropertyDescriptor(ID.NAME, "Name"));
		descriptorList.add(new ComboBoxPropertyDescriptor(ID.KIND, "Kind",
				new String[] { "STANDARD", "APPLYRULE", "FIRE", "SPLIT" }));
		descriptorList.add(new PropertyDescriptor(ID.VALID, "Valid"));
		descriptorList.add(new PropertyDescriptor(ID.ENABLED, "Enabled"));
		descriptorList.add(new TextPropertyDescriptor(ID.LOCATION, "Location"));
		return descriptorList.toArray(new IPropertyDescriptor[] {});
	}

	/*
	 * @see
	 * org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java
	 * .lang.Object)
	 */
	@Override
	public Object getPropertyValue(final Object id) {
		if (id instanceof ID) {
			switch ((ID) id) {
			case NAME:
				return getModel().getName();
			case KIND:
				return Integer.valueOf(getModel().getKind().getValue());
			case VALID:
				if (getModel().isSetValid()) {
					return String.valueOf(getModel().isValid());
				}
				return "unset";
			case ENABLED:
				if (getModel().isSetActivated()) {
					return String.valueOf(getModel().isActivated());
				}
				return "unset";
			case LOCATION:
				return new LocationPropertySource(getModel().getX(), getModel()
						.getY());
			}
		}
		return null;
	}

	/*
	 * @see
	 * org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java
	 * .lang.Object, java.lang.Object)
	 */
	@Override
	public void setPropertyValue(final Object id, final Object value) {
		switch ((ID) id) {
		case NAME:
			getModel().setName((String) value);
			break;
		case KIND:
			getModel().setKind(
					HLTransitionKind.get(((Integer) value).intValue()));
			break;
		case LOCATION:
			final Point newLoc = (Point) value;
			getModel().setX(newLoc.x);
			getModel().setY(newLoc.y);
			break;
		default:
		}
	}

}
