package roneditor.ui.dialogs;

import java.util.ArrayList;

import muvitorkit.dialogs.AbstractMatchDialog;
import muvitorkit.ui.ContextMenuProviderWithActionRegistry;

import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.requests.CreationFactory;

import roneditor.gef.editparts.objectnet.ObjectNetEditPartFactory;
import roneditor.model.ModelCreationFactory;
import roneditor.ui.rule.RulePageContextMenuProvider;
import ronmodel.Mapping;
import ronmodel.ObjectNet;
import ronmodel.Rule;

/**
 * Subclasses of AbstractMatchDialog are meant to adapt AbstractMatchDialog to a
 * concrete RONEditor by implementing the following methods.
 * 
 * @author Tony Modica
 */
public class MatchDialog extends AbstractMatchDialog {

	/**
	 * Main constructor for this dialog. Note that you should pass copies only,
	 * because the result mappings
	 * 
	 * @param rules
	 *            The rules that can be switched through for match-setting. Must
	 *            contain at least 1 rule.
	 * @param objectNets
	 *            The target object nets that can be switched through for
	 *            match-setting. Must contain at least 1 object net.
	 */
	public MatchDialog(final ArrayList<Rule> rules,
			final ArrayList<ObjectNet> objectNets) {
		super(rules, objectNets);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * roneditor.dialogs.match.AbstractMatchDialog#createContextMenuProvider
	 * (org.eclipse.gef.GraphicalViewer,
	 * org.eclipse.gef.ui.actions.ActionRegistry)
	 */
	@Override
	protected ContextMenuProviderWithActionRegistry createContextMenuProvider(
			final GraphicalViewer viewer) {
		return new RulePageContextMenuProvider(viewer);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.dialogs.match.AbstractMatchDialog#createEditPartFactory()
	 */
	@Override
	protected EditPartFactory createEditPartFactory() {
		return new ObjectNetEditPartFactory();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.dialogs.match.AbstractMatchDialog#createMappingTool()
	 */
	@Override
	protected CreationFactory createMappingCreationFactory() {
		return new ModelCreationFactory(Mapping.class);
	}
}
