package roneditor.ui.objectnet;

import muvitorkit.ui.MuvitorPageBookView;

import org.eclipse.core.runtime.Assert;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.ui.part.IPage;

import ronmodel.ObjectNet;
import ronmodel.RonmodelPackage;

/**
 * <p>
 * This class extends and implements the abstract class
 * {@link MuvitorPageBookView} to display a model on an {@link ObjectNetPage}. <br>
 * It is expected that the model object is a {@link ObjectNet} instance from the
 * {@link RonmodelPackage}.
 * </p>
 * 
 * <p>
 * If the EMF model notifies a change of the object net's name the views part
 * name is set to the value {@link #calculatePartName()} returns which is the
 * name of the model {@link ObjectNet}.
 * </p>
 * 
 * @author Tony Modica
 */
public class ObjectNetPageBookView extends MuvitorPageBookView {

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * roneditor.objectnetview.AbstractPageBookView#getPageForModel(java.lang
	 * .Object)
	 */
	@Override
	protected IPage createPageForModel(final EObject model) {
		Assert.isTrue(
				model instanceof ObjectNet,
				"The model that should be displayed in ObjectNetPageBookView is not an instance of the class ObjectNet!");
		return new ObjectNetPage(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * muvitorkit.ui.MuvitorPageBookView#notifyChanged(org.eclipse.emf.common
	 * .notify.Notification)
	 */
	@Override
	public void notifyChanged(final Notification notification) {
		final int featureId = notification.getFeatureID(RonmodelPackage.class);
		switch (featureId) {
		case RonmodelPackage.OBJECT_NET__NAME:
		case RonmodelPackage.OBJECT_NET__PLACE:
			setPartName(calculatePartName());
			break;
		default:
			break;
		}
	}

	/**
	 * @return The name of the model {@link ObjectNet}.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageBookView#calculatePartName()
	 */
	@Override
	protected String calculatePartName() {
		final ObjectNet net = (ObjectNet) getModel();
		if (net.getPlace() != null) {
			return net.getName() + "@" + net.getPlace().getName();
		}
		return net.getName();
	}

}
