package roneditor.ui.ron;

import muvitorkit.animation.demo.AnimatingDemoAction1;
import muvitorkit.animation.demo.AnimatingDemoAction2;
import muvitorkit.animation.demo.AnimatingDemoAction3;
import muvitorkit.gef.palette.MuvitorPaletteRoot;
import muvitorkit.ui.ContextMenuProviderWithActionRegistry;
import muvitorkit.ui.MuvitorPage;
import muvitorkit.ui.MuvitorPageBookView;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.KeyHandler;
import org.eclipse.ui.actions.ActionFactory;

import roneditor.gef.actions.ron.ApplyRuleDialogAction;
import roneditor.gef.actions.ron.FireDialogAction;
import roneditor.gef.actions.ron.FireHLTransitionAction;
import roneditor.gef.editparts.ron.RONEditPartFactory;
import aggSim.analyzer.ConflictAnalysisAction;
import aggSim.analyzer.LocalPlaceConflictAnalysisAction;

/**
 * @author Tony Modica
 */
public class RONPage extends MuvitorPage {

	public RONPage(final MuvitorPageBookView view) {
		super(view);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.ui.part.AbstractMultiViewerPage#createCustomActions()
	 */
	@Override
	protected void createCustomActions() {
		registerSharedActionAsHandler(ActionFactory.COPY.getId());
		registerSharedActionAsHandler(ActionFactory.CUT.getId());
		registerSharedActionAsHandler(ActionFactory.PASTE.getId());

		registerAction(new FireDialogAction(getEditor()));

		registerAction(new ApplyRuleDialogAction(getEditor()));

		// sample animation actions
		registerAction(new AnimatingDemoAction1(getEditor()));
		registerAction(new AnimatingDemoAction2(getEditor()));
		registerAction(new AnimatingDemoAction3(getEditor()));

		// analysis actions
		registerAction(new ConflictAnalysisAction(getEditor()));
		registerAction(new LocalPlaceConflictAnalysisAction(getEditor()));

		registerActionAsHandler(new FireHLTransitionAction(getEditor()));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * roneditor.objectnetview.AbstractPageWithFlyoutPalette#setupKeyHandler
	 * (org.eclipse.gef.KeyHandler)
	 */
	@Override
	protected void setupKeyHandler(final KeyHandler kh) {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * roneditor.objectnetview.AbstractPageWithFlyoutPalette#createEditPartFactory
	 * ()
	 */
	@Override
	protected EditPartFactory createEditPartFactory() {
		return new RONEditPartFactory();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * roneditor.objectnetview.AbstractPageWithFlyoutPalette#getContextMenuProvider
	 * ()
	 */
	@Override
	protected ContextMenuProviderWithActionRegistry createContextMenuProvider(
			final EditPartViewer viewer) {
		return new RONEditorContextMenuProvider(viewer);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * roneditor.objectnetview.AbstractPageWithFlyoutPalette#createPaletteRoot()
	 */
	@Override
	protected MuvitorPaletteRoot createPaletteRoot() {
		return new RONPaletteRoot();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.ruleview.AbstractMultiViewerPage#getViewerContents()
	 */
	@Override
	public EObject[] getViewerContents() {
		return new EObject[] { getModel() };
	}
}
