package roneditor.ui.wizards;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;

public class CreateHLPlaceWizardPage extends WizardPage implements Listener {

	private Button ruleRadioButton;

	private Button objectNetRadioButton;

	private Text placeName;

	protected CreateHLPlaceWizardPage(final IWorkbench workbench,
			final IStructuredSelection selection) {
		super("High-Level Place");
		setTitle("High-Level Place");
		setDescription("Create a new High-Level Place");
	}

	@Override
	public void createControl(final Composite parent) {
		GridData gd;
		final Composite composite = new Composite(parent, SWT.NULL);

		final GridLayout gl = new GridLayout();
		final int ncol = 3;
		gl.numColumns = ncol;
		composite.setLayout(gl);

		// Type Radio Buttons
		new Label(composite, SWT.NONE).setText("Type:");
		objectNetRadioButton = new Button(composite, SWT.RADIO);
		objectNetRadioButton.setText("Object Network");
		objectNetRadioButton.setSelection(true);
		ruleRadioButton = new Button(composite, SWT.RADIO);
		ruleRadioButton.setText("Rule");

		// Name Text Field
		new Label(composite, SWT.NONE).setText("Name:");
		placeName = new Text(composite, SWT.BORDER);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = ncol - 1;
		placeName.setLayoutData(gd);

		placeName.setFocus();

		setControl(composite);
		addListeners();
	}

	@Override
	public boolean isPageComplete() {
		final CreateHLPlaceWizard wizard = (CreateHLPlaceWizard) getWizard();
		if (placeName.getText().isEmpty()) {
			return false;
		}
		wizard.setPlaceName(placeName.getText());
		wizard.setRule(ruleRadioButton.getSelection());
		return true;
	}

	private void addListeners() {
		objectNetRadioButton.addListener(SWT.Selection, this);
		ruleRadioButton.addListener(SWT.Selection, this);
		placeName.addListener(SWT.KeyUp, this);
	}

	@Override
	public void handleEvent(final Event event) {
		if (placeName.getText().isEmpty()) {
			setErrorMessage("Name of the Place is empty");
		} else {
			setErrorMessage(null);
		}
		getWizard().getContainer().updateButtons();
	}

}
